from _typeshed import Incomplete
from collections.abc import Callable, Sequence
from logging import Logger
from typing import Any, Literal, TypeVar
from typing_extensions import TypeAlias, deprecated

from django.contrib import admin
from django.contrib.admin.helpers import ActionForm
from django.core.files import File
from django.db.models import Model, QuerySet
from django.forms import Form
from django.http.request import HttpRequest
from django.http.response import HttpResponse
from django.template.response import TemplateResponse
from django.urls import URLPattern

from .formats.base_formats import Format
from .mixins import BaseExportMixin, BaseImportMixin
from .results import Result
from .tmp_storages import BaseStorage

Dataset: TypeAlias = Incomplete  # tablib.Dataset
logger: Logger

_ModelT = TypeVar("_ModelT", bound=Model)

class ImportExportMixinBase:
    base_change_list_template: str
    change_list_template: str
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def init_change_list_template(self) -> None: ...
    def get_model_info(self) -> tuple[str, str]: ...
    def changelist_view(self, request: HttpRequest, extra_context: dict[str, Any] | None = None) -> HttpResponse: ...

class ImportMixin(BaseImportMixin[_ModelT], ImportExportMixinBase):
    import_export_change_list_template: str
    import_template_name: str
    import_form_class: type[Form] = ...
    confirm_form_class: type[Form] = ...
    from_encoding: str
    import_error_display: Sequence[Literal["message", "row", "traceback"]]
    skip_admin_log: bool | None
    tmp_storage_class: str | type[BaseStorage]
    def get_skip_admin_log(self) -> bool: ...
    def get_tmp_storage_class(self) -> type[BaseStorage]: ...
    def get_tmp_storage_class_kwargs(self) -> dict[str, Any]: ...
    def has_import_permission(self, request: HttpRequest) -> bool: ...
    def get_urls(self) -> list[URLPattern]: ...
    def process_import(self, request: HttpRequest, **kwargs: Any) -> HttpResponse: ...
    def process_dataset(self, dataset: Dataset, form: Form, request: HttpRequest, **kwargs: Any) -> Result: ...
    def process_result(self, result: Result, request: HttpRequest) -> HttpResponse: ...
    def generate_log_entries(self, result: Result, request: HttpRequest) -> None: ...
    def add_success_message(self, result: Result, request: HttpRequest) -> None: ...
    def get_import_context_data(self, **kwargs: Any) -> dict[str, Any]: ...
    def get_context_data(self, **kwargs: Any) -> dict[str, Any]: ...
    def create_import_form(self, request: HttpRequest) -> Form: ...
    def get_import_form_class(self, request: HttpRequest) -> type[Form]: ...
    def get_import_form_kwargs(self, request: HttpRequest) -> dict[str, Any]: ...
    def get_import_form_initial(self, request: HttpRequest) -> dict[str, Any]: ...
    def create_confirm_form(self, request: HttpRequest, import_form: Form | None = None) -> Form: ...
    def get_confirm_form_class(self, request: HttpRequest) -> type[Form]: ...
    def get_confirm_form_kwargs(self, request: HttpRequest, import_form: Form | None = None) -> dict[str, Any]: ...
    def get_confirm_form_initial(self, request: HttpRequest, import_form: Form | None) -> dict[str, Any]: ...
    def get_import_data_kwargs(self, **kwargs: Any) -> dict[str, Any]: ...
    def write_to_tmp_storage(self, import_file: File[bytes], input_format: Format) -> BaseStorage: ...
    def add_data_read_fail_error_to_form(self, form: Form, e: Exception) -> None: ...
    def import_action(self, request: HttpRequest, **kwargs: Any) -> TemplateResponse: ...
    def changelist_view(self, request: HttpRequest, extra_context: dict[str, Any] | None = None) -> HttpResponse: ...

class ExportMixin(BaseExportMixin[_ModelT], ImportExportMixinBase):
    import_export_change_list_template: str
    export_template_name: str
    to_encoding: str | None
    export_form_class: type[Form] = ...
    def get_urls(self) -> list[URLPattern]: ...
    def has_export_permission(self, request: HttpRequest) -> bool: ...
    def get_export_queryset(self, request: HttpRequest) -> QuerySet[_ModelT]: ...
    def get_export_data(
        self, file_format: Format, request: HttpRequest, queryset: QuerySet[_ModelT], **kwargs: Any
    ) -> str | bytes: ...
    def get_export_context_data(self, **kwargs: Any) -> dict[str, Any]: ...
    def get_context_data(self, **kwargs: Any) -> dict[str, Any]: ...
    def get_export_form_class(self) -> type[Form]: ...
    def export_action(self, request: HttpRequest) -> TemplateResponse: ...
    @deprecated(
        "The 'get_valid_export_item_pks()' method is deprecated and will be removed in a future release. "
        "Overwrite 'get_queryset()' or 'get_export_queryset()' instead."
    )
    def get_valid_export_item_pks(self, request: HttpRequest) -> list[str]: ...
    def changelist_view(self, request: HttpRequest, extra_context: dict[str, Any] | None = None) -> HttpResponse: ...
    def get_export_filename(self, request: HttpRequest, queryset: QuerySet[_ModelT], file_format: Format) -> str: ...  # type: ignore[override]
    def init_request_context_data(self, request: HttpRequest, form: Form) -> dict[str, Any]: ...

class ImportExportMixin(ImportMixin[_ModelT], ExportMixin[_ModelT]): ...
class ImportExportModelAdmin(ImportExportMixin[_ModelT], admin.ModelAdmin[_ModelT]): ...  # type: ignore[misc]

class ExportActionMixin(ExportMixin[_ModelT]):
    change_form_template: str
    show_change_form_export: bool
    action_form: type[ActionForm]
    def change_view(
        self, request: HttpRequest, object_id: str, form_url: str = "", extra_context: dict[str, Any] | None = None
    ) -> HttpResponse: ...
    def response_change(self, request: HttpRequest, obj: _ModelT) -> HttpResponse: ...
    def export_admin_action(self, request: HttpRequest, queryset: QuerySet[_ModelT]) -> HttpResponse: ...
    def get_actions(self, request: HttpRequest) -> dict[str, tuple[Callable[..., str], str, str] | None]: ...

class ExportActionModelAdmin(ExportActionMixin[_ModelT], admin.ModelAdmin[_ModelT]): ...  # type: ignore[misc]
class ImportExportActionModelAdmin(ImportMixin[_ModelT], ExportActionModelAdmin[_ModelT]): ...  # type: ignore[misc]



from asgiref.server import StatelessServer
from channels.layers import BaseChannelLayer
from channels.utils import _ChannelApplication

class Worker(StatelessServer):
    channels: list[str]
    channel_layer: BaseChannelLayer

    def __init__(
        self, application: _ChannelApplication, channels: list[str], channel_layer: BaseChannelLayer, max_applications: int = 1000
    ) -> None: ...
    async def handle(self) -> None: ...
    async def listener(self, channel: str) -> None: ...



import enum
import functools
from _typeshed import Unused
from collections.abc import Iterable, Mapping
from typing import Any, Literal

from ..extractor.common import _InfoDict
from .fragment import FragmentFD

class Features(enum.Enum):
    TO_STDOUT = 1
    MULTIPLE_FORMATS = 2

class ExternalFD(FragmentFD):
    SUPPORTED_PROTOCOLS: tuple[str, ...]
    SUPPORTED_FEATURES: tuple[Features, ...]
    @classmethod
    def get_basename(cls) -> str: ...
    @property
    def EXE_NAME(cls) -> str: ...
    @functools.cached_property
    def exe(self) -> str: ...
    @classmethod
    def available(cls, path: str | None = None) -> str | Literal[False]: ...
    @classmethod
    def supports(cls, info_dict: _InfoDict) -> bool: ...
    @classmethod
    def can_download(cls, info_dict: _InfoDict, path: str | None = None) -> bool: ...

class CurlFD(ExternalFD):
    AVAILABLE_OPT: str

class AxelFD(ExternalFD):
    AVAILABLE_OPT: str

class WgetFD(ExternalFD):
    AVAILABLE_OPT: str

class Aria2cFD(ExternalFD):
    AVAILABLE_OPT: str
    SUPPORTED_PROTOCOLS: tuple[str, ...]
    @staticmethod
    def supports_manifest(manifest: str) -> bool: ...
    def aria2c_rpc(self, rpc_port: int, rpc_secret: str, method: str, params: Iterable[str] = ()) -> Any: ...

class HttpieFD(ExternalFD):
    AVAILABLE_OPT: str
    EXE_NAME: str

class FFmpegFD(ExternalFD):
    SUPPORTED_PROTOCOLS: tuple[str, ...]
    SUPPORTED_FEATURES: tuple[Features, ...]
    @classmethod
    def available(cls, path: str | None = None) -> bool: ...  # type: ignore[override]
    def on_process_started(self, proc: Unused, stdin: Unused) -> None: ...
    @classmethod
    def can_merge_formats(cls, info_dict: _InfoDict, params: Mapping[str, Any]) -> bool: ...

def list_external_downloaders() -> list[str]: ...
def get_external_downloader(external_downloader: str) -> type[ExternalFD]: ...



import datetime

__all__ = ["tzid_from_tzinfo", "tzid_from_dt", "tzids_from_tzinfo"]

def tzids_from_tzinfo(tzinfo: datetime.tzinfo | None) -> tuple[str, ...]: ...
def tzid_from_tzinfo(tzinfo: datetime.tzinfo | None) -> str | None: ...
def tzid_from_dt(dt: datetime.datetime) -> str | None: ...



import struct
from _typeshed import SupportsWrite
from collections.abc import Collection, Mapping
from typing import Any, Final

from .fragment import FragmentFD

u8: Final[struct.Struct]
u88: Final[struct.Struct]
u16: Final[struct.Struct]
u1616: Final[struct.Struct]
u32: Final[struct.Struct]
u64: Final[struct.Struct]
s88: Final[struct.Struct]
s16: Final[struct.Struct]
s1616: Final[struct.Struct]
s32: Final[struct.Struct]
unity_matrix: Final[bytes]
TRACK_ENABLED: Final = 0x1
TRACK_IN_MOVIE: Final = 0x2
TRACK_IN_PREVIEW: Final = 0x4
SELF_CONTAINED: Final = 0x1

def box(box_type: bytes, payload: bytes) -> bytes: ...
def full_box(box_type: bytes, version: int, flags: int, payload: bytes) -> bytes: ...
def write_piff_header(stream: SupportsWrite[bytes], params: Mapping[str, Any]) -> None: ...
def extract_box_data(data: bytes, box_sequence: Collection[bytes]) -> bytes | None: ...

class IsmFD(FragmentFD): ...



import argparse
from _typeshed import ExcInfo, FileDescriptorOrPath, Incomplete, OptExcInfo, SupportsWrite
from collections.abc import Mapping, Sequence
from typing import Any
from typing_extensions import Self, deprecated

@deprecated("The jsonschema CLI is deprecated and will be removed in a future version. Please use check-jsonschema instead.")
class _CannotLoadFile(Exception): ...

@deprecated("The jsonschema CLI is deprecated and will be removed in a future version. Please use check-jsonschema instead.")
class _Outputter:
    __slots__ = ("_formatter", "_stdout", "_stderr", "__weakref__")
    _formatter: _PlainFormatter | _PrettyFormatter
    _stdout: SupportsWrite[str]
    _stderr: SupportsWrite[str]

    def __init__(
        self, formatter: _PlainFormatter | _PrettyFormatter, stdout: SupportsWrite[str], stderr: SupportsWrite[str]
    ) -> None: ...
    @classmethod
    def from_arguments(
        cls, arguments: Mapping[str, Incomplete], stdout: SupportsWrite[str], stderr: SupportsWrite[str]
    ) -> Self: ...
    def load(self, path: FileDescriptorOrPath) -> Any: ...  # result of json.load()
    def filenotfound_error(self, *, path: FileDescriptorOrPath, exc_info: OptExcInfo) -> None: ...
    def parsing_error(self, *, path: FileDescriptorOrPath, exc_info: ExcInfo) -> None: ...
    def validation_error(self, *, instance_path: FileDescriptorOrPath, error: BaseException) -> None: ...
    def validation_success(self, *, instance_path: FileDescriptorOrPath) -> None: ...

@deprecated("The jsonschema CLI is deprecated and will be removed in a future version. Please use check-jsonschema instead.")
class _PrettyFormatter:
    __slots__ = ("__weakref__",)
    _ERROR_MSG: str
    _SUCCESS_MSG: str

    def __init__(self) -> None: ...
    def filenotfound_error(self, path: FileDescriptorOrPath, exc_info: OptExcInfo) -> str: ...
    def parsing_error(self, path: FileDescriptorOrPath, exc_info: ExcInfo) -> str: ...
    def validation_error(self, instance_path: FileDescriptorOrPath, error: BaseException) -> str: ...
    def validation_success(self, instance_path: FileDescriptorOrPath) -> str: ...

@deprecated("The jsonschema CLI is deprecated and will be removed in a future version. Please use check-jsonschema instead.")
class _PlainFormatter:
    __slots__ = ("_error_format", "__weakref__")
    _error_format: str

    def __init__(self, error_format: str) -> None: ...
    def filenotfound_error(self, path: FileDescriptorOrPath, exc_info: OptExcInfo) -> str: ...
    def parsing_error(self, path: FileDescriptorOrPath, exc_info: ExcInfo) -> str: ...
    def validation_error(self, instance_path: FileDescriptorOrPath, error: BaseException) -> str: ...
    def validation_success(self, instance_path: FileDescriptorOrPath) -> str: ...

parser: argparse.ArgumentParser

@deprecated("The jsonschema CLI is deprecated and will be removed in a future version. Please use check-jsonschema instead.")
def parse_args(args: Sequence[str] | None) -> dict[str, Any]: ...  # result of vars(argparse.Namespace())
@deprecated("The jsonschema CLI is deprecated and will be removed in a future version. Please use check-jsonschema instead.")
def main(args: Sequence[str] = ...) -> None: ...
@deprecated("The jsonschema CLI is deprecated and will be removed in a future version. Please use check-jsonschema instead.")
def run(
    arguments: Mapping[str, Incomplete],
    stdout: SupportsWrite[str] = ...,
    stderr: SupportsWrite[str] = ...,
    stdin: SupportsWrite[str] = ...,
) -> int: ...



import enum
from collections.abc import Callable
from typing import Any, AnyStr, Generic
from typing_extensions import TypeAlias

from ._main import Pattern

__all__ = [
    "A",
    "ASCII",
    "B",
    "BESTMATCH",
    "D",
    "DEBUG",
    "E",
    "ENHANCEMATCH",
    "F",
    "FULLCASE",
    "I",
    "IGNORECASE",
    "L",
    "LOCALE",
    "M",
    "MULTILINE",
    "P",
    "POSIX",
    "R",
    "REVERSE",
    "S",
    "DOTALL",
    "T",
    "TEMPLATE",
    "U",
    "UNICODE",
    "V0",
    "VERSION0",
    "V1",
    "VERSION1",
    "W",
    "WORD",
    "X",
    "VERBOSE",
    "error",
    "Scanner",
    "RegexFlag",
]

class error(Exception):
    def __init__(self, message: str, pattern: AnyStr | None = None, pos: int | None = None) -> None: ...

class RegexFlag(enum.IntFlag):
    A = 0x80
    ASCII = A
    B = 0x1000
    BESTMATCH = B
    D = 0x200
    DEBUG = D
    E = 0x8000
    ENHANCEMATCH = E
    F = 0x4000
    FULLCASE = F
    I = 0x2
    IGNORECASE = I
    L = 0x4
    LOCALE = L
    M = 0x8
    MULTILINE = M
    P = 0x10000
    POSIX = P
    R = 0x400
    REVERSE = R
    T = 0x1
    TEMPLATE = T
    S = 0x10
    DOTALL = S
    U = 0x20
    UNICODE = U
    V0 = 0x2000
    VERSION0 = V0
    V1 = 0x100
    VERSION1 = V1
    W = 0x800
    WORD = W
    X = 0x40
    VERBOSE = X

ASCII = RegexFlag.ASCII
BESTMATCH = RegexFlag.BESTMATCH
DEBUG = RegexFlag.DEBUG
ENHANCEMATCH = RegexFlag.ENHANCEMATCH
FULLCASE = RegexFlag.FULLCASE
IGNORECASE = RegexFlag.IGNORECASE
LOCALE = RegexFlag.LOCALE
MULTILINE = RegexFlag.MULTILINE
POSIX = RegexFlag.POSIX
REVERSE = RegexFlag.REVERSE
TEMPLATE = RegexFlag.TEMPLATE
DOTALL = RegexFlag.DOTALL
UNICODE = RegexFlag.UNICODE
VERBOSE = RegexFlag.VERBOSE
VERSION0 = RegexFlag.VERSION0
VERSION1 = RegexFlag.VERSION1
WORD = RegexFlag.WORD
A = RegexFlag.A
B = RegexFlag.B
D = RegexFlag.D
E = RegexFlag.E
F = RegexFlag.F
I = RegexFlag.I
L = RegexFlag.L
M = RegexFlag.M
P = RegexFlag.P
R = RegexFlag.R
S = RegexFlag.S
U = RegexFlag.U
V0 = RegexFlag.V0
V1 = RegexFlag.V1
W = RegexFlag.W
X = RegexFlag.X
T = RegexFlag.T

DEFAULT_VERSION = VERSION1

_Lexicon: TypeAlias = list[tuple[AnyStr, Callable[[Scanner[AnyStr], AnyStr], Any]]]

class Scanner(Generic[AnyStr]):
    lexicon: _Lexicon[AnyStr]
    scanner: Pattern[AnyStr]

    def __init__(self, lexicon: _Lexicon[AnyStr], flags: int = 0) -> None: ...
    def scan(self, string: AnyStr) -> tuple[list[Any], AnyStr]: ...



from . import croniter as croniter_m
from .croniter import (
    DAY_FIELD as DAY_FIELD,
    HOUR_FIELD as HOUR_FIELD,
    MINUTE_FIELD as MINUTE_FIELD,
    MONTH_FIELD as MONTH_FIELD,
    OVERFLOW32B_MODE as OVERFLOW32B_MODE,
    SECOND_FIELD as SECOND_FIELD,
    UTC_DT as UTC_DT,
    YEAR_FIELD as YEAR_FIELD,
    CroniterBadCronError as CroniterBadCronError,
    CroniterBadDateError as CroniterBadDateError,
    CroniterBadTypeRangeError as CroniterBadTypeRangeError,
    CroniterError as CroniterError,
    CroniterNotAlphaError as CroniterNotAlphaError,
    CroniterUnsupportedSyntaxError as CroniterUnsupportedSyntaxError,
    croniter as croniter,
    croniter_range as croniter_range,
    datetime_to_timestamp as datetime_to_timestamp,
)

cron_m = croniter_m



from collections.abc import Iterable
from typing import Literal, TypedDict, type_check_only

from channels.testing.application import ApplicationCommunicator
from channels.utils import _ChannelApplication

# HTTP test-specific response type
@type_check_only
class _HTTPTestResponse(TypedDict, total=False):
    status: int
    headers: Iterable[tuple[bytes, bytes]]
    body: bytes

@type_check_only
class _HTTPTestScope(TypedDict, total=False):
    type: Literal["http"]
    http_version: str
    method: str
    scheme: str
    path: str
    raw_path: bytes
    query_string: bytes
    root_path: str
    headers: Iterable[tuple[bytes, bytes]] | None
    client: tuple[str, int] | None
    server: tuple[str, int | None] | None

class HttpCommunicator(ApplicationCommunicator):
    scope: _HTTPTestScope
    body: bytes
    sent_request: bool

    def __init__(
        self,
        application: _ChannelApplication,
        method: str,
        path: str,
        body: bytes = b"",
        headers: Iterable[tuple[bytes, bytes]] | None = None,
    ) -> None: ...
    async def get_response(self, timeout: float = 1) -> _HTTPTestResponse: ...



from collections.abc import Callable, Iterable
from typing import Any, Literal, overload

from ._typing import GeoT, OptGeoArrayLike, SupportsGeoInterface
from .algorithms.polylabel import polylabel as polylabel
from .geometry import GeometryCollection, LineString, MultiLineString, Point, Polygon
from .geometry.base import BaseGeometry, BaseMultipartGeometry, GeometrySequence
from .geometry.linestring import _ConvertibleToLineString
from .lib import Geometry

__all__ = [
    "clip_by_rect",
    "linemerge",
    "nearest_points",
    "operator",
    "orient",
    "polygonize",
    "polygonize_full",
    "shared_paths",
    "snap",
    "split",
    "substring",
    "transform",
    "triangulate",
    "unary_union",
    "validate",
    "voronoi_diagram",
]

class CollectionOperator:
    @overload
    def shapeup(self, ob: GeoT) -> GeoT: ...  # type: ignore[overload-overlap]
    @overload
    def shapeup(self, ob: dict[str, Any] | SupportsGeoInterface) -> BaseGeometry: ...  # type: ignore[overload-overlap]
    @overload
    def shapeup(self, ob: _ConvertibleToLineString) -> LineString: ...
    def polygonize(
        self, lines: OptGeoArrayLike | Iterable[_ConvertibleToLineString | None]
    ) -> GeometrySequence[GeometryCollection[Polygon]]: ...
    def polygonize_full(
        self, lines: OptGeoArrayLike | Iterable[_ConvertibleToLineString | None]
    ) -> tuple[
        GeometryCollection[Polygon], GeometryCollection[LineString], GeometryCollection[LineString], GeometryCollection[Polygon]
    ]: ...
    def linemerge(
        self, lines: MultiLineString | BaseMultipartGeometry | Iterable[_ConvertibleToLineString], directed: bool = False
    ) -> LineString | MultiLineString: ...
    def unary_union(self, geoms: OptGeoArrayLike) -> BaseGeometry: ...

operator: CollectionOperator
polygonize = operator.polygonize
polygonize_full = operator.polygonize_full
linemerge = operator.linemerge
unary_union = operator.unary_union

# This is also an alias to operator method but we want to mark it as deprecated
@overload  # edges false
def triangulate(geom: Geometry, tolerance: float = 0.0, edges: Literal[False] = False) -> list[Polygon]: ...
@overload  # edges true (keyword)
def triangulate(geom: Geometry, tolerance: float = 0.0, *, edges: Literal[True]) -> list[LineString]: ...
@overload  # edges true (positional)
def triangulate(geom: Geometry, tolerance: float, edges: Literal[True]) -> list[LineString]: ...
@overload  # fallback
def triangulate(geom: Geometry, tolerance: float = 0.0, edges: bool = False) -> list[Polygon] | list[LineString]: ...
@overload
def voronoi_diagram(
    geom: Geometry, envelope: Geometry | None = None, tolerance: float = 0.0, edges: Literal[False] = False
) -> GeometryCollection[Polygon]: ...
@overload
def voronoi_diagram(
    geom: Geometry, envelope: Geometry | None, tolerance: float, edges: Literal[True]
) -> GeometryCollection[LineString | MultiLineString]: ...
@overload
def voronoi_diagram(
    geom: Geometry, envelope: Geometry | None = None, tolerance: float = 0.0, *, edges: Literal[True]
) -> GeometryCollection[LineString | MultiLineString]: ...
@overload
def voronoi_diagram(
    geom: Geometry, envelope: Geometry | None = None, tolerance: float = 0.0, edges: bool = False
) -> GeometryCollection[Polygon | LineString | MultiLineString]: ...
@overload
def validate(geom: None) -> None: ...
@overload
def validate(geom: Geometry) -> str: ...
@overload
def validate(geom: Geometry | None) -> str | None: ...
def transform(func: Callable[[float, float, float | None], tuple[float, ...]], geom: GeoT) -> GeoT: ...
def nearest_points(g1: Geometry, g2: Geometry) -> tuple[Point, Point]: ...
def snap(g1: GeoT, g2: Geometry, tolerance: float) -> GeoT: ...
def shared_paths(g1: LineString, g2: LineString) -> GeometryCollection[MultiLineString]: ...

class SplitOp:
    @staticmethod
    def split(geom: Geometry, splitter: Geometry) -> GeometryCollection: ...

split = SplitOp.split

def substring(geom: LineString, start_dist: float, end_dist: float, normalized: bool = False) -> Point | LineString: ...
def clip_by_rect(geom: Geometry, xmin: float, ymin: float, xmax: float, ymax: float) -> BaseGeometry: ...
def orient(geom: GeoT, sign: float = 1.0) -> GeoT: ...



from decimal import Decimal
from typing import Literal, TypedDict, type_check_only

@type_check_only
class _TimeEstimate(TypedDict):
    crack_times_seconds: _CrackTimeSeconds
    crack_times_display: _CrackTimesDisplay
    score: Literal[0, 1, 2, 3, 4]

@type_check_only
class _CrackTimeSeconds(TypedDict):
    online_throttling_100_per_hour: Decimal
    online_no_throttling_10_per_second: Decimal
    offline_slow_hashing_1e4_per_second: Decimal
    offline_fast_hashing_1e10_per_second: Decimal

@type_check_only
class _CrackTimesDisplay(TypedDict):
    online_throttling_100_per_hour: str
    online_no_throttling_10_per_second: str
    offline_slow_hashing_1e4_per_second: str
    offline_fast_hashing_1e10_per_second: str

def estimate_attack_times(guesses: Decimal | float) -> _TimeEstimate: ...
def guesses_to_score(guesses: Decimal) -> Literal[0, 1, 2, 3, 4]: ...
def display_time(seconds: float) -> str: ...
def float_to_decimal(f: float) -> Decimal: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class SaverDef(google.protobuf.message.Message):
    """Protocol buffer representing the configuration of a Saver."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _CheckpointFormatVersion:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _CheckpointFormatVersionEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[SaverDef._CheckpointFormatVersion.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        LEGACY: SaverDef._CheckpointFormatVersion.ValueType  # 0
        """Internal legacy format."""
        V1: SaverDef._CheckpointFormatVersion.ValueType  # 1
        """Deprecated format: tf.Saver() which works with tensorflow::table::Table."""
        V2: SaverDef._CheckpointFormatVersion.ValueType  # 2
        """Current format: more efficient."""

    class CheckpointFormatVersion(_CheckpointFormatVersion, metaclass=_CheckpointFormatVersionEnumTypeWrapper):
        """A version number that identifies a different on-disk checkpoint format.
        Usually, each subclass of BaseSaverBuilder works with a particular
        version/format.  However, it is possible that the same builder may be
        upgraded to support a newer checkpoint format in the future.
        """

    LEGACY: SaverDef.CheckpointFormatVersion.ValueType  # 0
    """Internal legacy format."""
    V1: SaverDef.CheckpointFormatVersion.ValueType  # 1
    """Deprecated format: tf.Saver() which works with tensorflow::table::Table."""
    V2: SaverDef.CheckpointFormatVersion.ValueType  # 2
    """Current format: more efficient."""

    FILENAME_TENSOR_NAME_FIELD_NUMBER: builtins.int
    SAVE_TENSOR_NAME_FIELD_NUMBER: builtins.int
    RESTORE_OP_NAME_FIELD_NUMBER: builtins.int
    MAX_TO_KEEP_FIELD_NUMBER: builtins.int
    SHARDED_FIELD_NUMBER: builtins.int
    KEEP_CHECKPOINT_EVERY_N_HOURS_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    filename_tensor_name: builtins.str
    """The name of the tensor in which to specify the filename when saving or
    restoring a model checkpoint.
    """
    save_tensor_name: builtins.str
    """The operation to run when saving a model checkpoint."""
    restore_op_name: builtins.str
    """The operation to run when restoring a model checkpoint."""
    max_to_keep: builtins.int
    """Maximum number of checkpoints to keep.  If 0, no checkpoints are deleted."""
    sharded: builtins.bool
    """Shard the save files, one per device that has Variable nodes."""
    keep_checkpoint_every_n_hours: builtins.float
    """How often to keep an additional checkpoint. If not specified, only the last
    "max_to_keep" checkpoints are kept; if specified, in addition to keeping
    the last "max_to_keep" checkpoints, an additional checkpoint will be kept
    for every n hours of training.
    """
    version: global___SaverDef.CheckpointFormatVersion.ValueType
    def __init__(
        self,
        *,
        filename_tensor_name: builtins.str | None = ...,
        save_tensor_name: builtins.str | None = ...,
        restore_op_name: builtins.str | None = ...,
        max_to_keep: builtins.int | None = ...,
        sharded: builtins.bool | None = ...,
        keep_checkpoint_every_n_hours: builtins.float | None = ...,
        version: global___SaverDef.CheckpointFormatVersion.ValueType | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "filename_tensor_name",
            b"filename_tensor_name",
            "keep_checkpoint_every_n_hours",
            b"keep_checkpoint_every_n_hours",
            "max_to_keep",
            b"max_to_keep",
            "restore_op_name",
            b"restore_op_name",
            "save_tensor_name",
            b"save_tensor_name",
            "sharded",
            b"sharded",
            "version",
            b"version",
        ],
    ) -> None: ...

global___SaverDef = SaverDef



from typing import AnyStr

from .base import ContentLine
from .behavior import Behavior

class Name:
    family: str | list[str]
    given: str | list[str]
    additional: str | list[str]
    prefix: str | list[str]
    suffix: str | list[str]
    def __init__(
        self,
        family: str | list[str] = "",
        given: str | list[str] = "",
        additional: str | list[str] = "",
        prefix: str | list[str] = "",
        suffix: str | list[str] = "",
    ) -> None: ...
    @staticmethod
    def toString(val: str | list[str] | tuple[str, ...]) -> str: ...
    def __eq__(self, other: object) -> bool: ...

class Address:
    box: str | list[str]
    extended: str | list[str]
    street: str | list[str]
    city: str | list[str]
    region: str | list[str]
    code: str | list[str]
    country: str | list[str]
    def __init__(
        self,
        street: str | list[str] = "",
        city: str | list[str] = "",
        region: str | list[str] = "",
        code: str | list[str] = "",
        country: str | list[str] = "",
        box: str | list[str] = "",
        extended: str | list[str] = "",
    ) -> None: ...
    @staticmethod
    def toString(val: str | list[str] | tuple[str, ...], join_char: str = "\n") -> str: ...
    lines: tuple[str, ...]
    one_line: tuple[str, ...]
    def __eq__(self, other: object) -> bool: ...

class VCardTextBehavior(Behavior):
    allowGroup: bool
    base64string: str
    @classmethod
    def decode(cls, line: ContentLine) -> None: ...
    @classmethod
    def encode(cls, line: ContentLine) -> None: ...

class VCardBehavior(Behavior):
    allowGroup: bool
    defaultBehavior: type[VCardTextBehavior]

class VCard3_0(VCardBehavior):
    name: str
    description: str
    versionString: str
    isComponent: bool
    sortFirst: tuple[str, ...]
    @classmethod
    def generateImplicitParameters(cls, obj) -> None: ...

class FN(VCardTextBehavior):
    name: str
    description: str

class Label(VCardTextBehavior):
    name: str
    description: str

class GEO(VCardBehavior): ...

wacky_apple_photo_serialize: bool
REALLY_LARGE: float

class Photo(VCardTextBehavior):
    name: str
    description: str
    @classmethod
    def valueRepr(cls, line: ContentLine) -> str: ...
    @classmethod
    def serialize(cls, obj, buf, lineLength, validate, *args, **kwargs) -> None: ...  # type: ignore[override]

def toListOrString(string: str) -> str | list[str]: ...
def splitFields(string: str) -> list[str | list[str]]: ...
def toList(stringOrList: AnyStr | list[AnyStr]) -> list[AnyStr]: ...
def serializeFields(obj, order=None): ...

NAME_ORDER: tuple[str, ...]
ADDRESS_ORDER: tuple[str, ...]

class NameBehavior(VCardBehavior):
    hasNative: bool
    @staticmethod
    def transformToNative(obj): ...
    @staticmethod
    def transformFromNative(obj): ...

class AddressBehavior(VCardBehavior):
    hasNative: bool
    @staticmethod
    def transformToNative(obj): ...
    @staticmethod
    def transformFromNative(obj): ...

class OrgBehavior(VCardBehavior):
    hasNative: bool
    @staticmethod
    def transformToNative(obj): ...
    @staticmethod
    def transformFromNative(obj): ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _DataType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DataTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DataType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    DT_INVALID: _DataType.ValueType  # 0
    """Not a legal value for DataType.  Used to indicate a DataType field
    has not been set.
    """
    DT_FLOAT: _DataType.ValueType  # 1
    """Data types that all computation devices are expected to be
    capable to support.
    """
    DT_DOUBLE: _DataType.ValueType  # 2
    DT_INT32: _DataType.ValueType  # 3
    DT_UINT8: _DataType.ValueType  # 4
    DT_INT16: _DataType.ValueType  # 5
    DT_INT8: _DataType.ValueType  # 6
    DT_STRING: _DataType.ValueType  # 7
    DT_COMPLEX64: _DataType.ValueType  # 8
    """Single-precision complex"""
    DT_INT64: _DataType.ValueType  # 9
    DT_BOOL: _DataType.ValueType  # 10
    DT_QINT8: _DataType.ValueType  # 11
    """Quantized int8"""
    DT_QUINT8: _DataType.ValueType  # 12
    """Quantized uint8"""
    DT_QINT32: _DataType.ValueType  # 13
    """Quantized int32"""
    DT_BFLOAT16: _DataType.ValueType  # 14
    """Float32 truncated to 16 bits."""
    DT_QINT16: _DataType.ValueType  # 15
    """Quantized int16"""
    DT_QUINT16: _DataType.ValueType  # 16
    """Quantized uint16"""
    DT_UINT16: _DataType.ValueType  # 17
    DT_COMPLEX128: _DataType.ValueType  # 18
    """Double-precision complex"""
    DT_HALF: _DataType.ValueType  # 19
    DT_RESOURCE: _DataType.ValueType  # 20
    DT_VARIANT: _DataType.ValueType  # 21
    """Arbitrary C++ data types"""
    DT_UINT32: _DataType.ValueType  # 22
    DT_UINT64: _DataType.ValueType  # 23
    DT_FLOAT8_E5M2: _DataType.ValueType  # 24
    """5 exponent bits, 2 mantissa bits."""
    DT_FLOAT8_E4M3FN: _DataType.ValueType  # 25
    """4 exponent bits, 3 mantissa bits, finite-only, with"""
    DT_INT4: _DataType.ValueType  # 29
    """2 NaNs (0bS1111111).
    TODO - b/299182407: Leaving room for remaining float8 types.
    DT_FLOAT8_E4M3FNUZ = 26;
    DT_FLOAT8_E4M3B11FNUZ = 27;
    DT_FLOAT8_E5M2FNUZ = 28;
    """
    DT_UINT4: _DataType.ValueType  # 30
    DT_FLOAT_REF: _DataType.ValueType  # 101
    """Do not use!  These are only for TF1's obsolete reference Variables.
    Every enum above should have a corresponding value below (verified by
    types_test).
    """
    DT_DOUBLE_REF: _DataType.ValueType  # 102
    DT_INT32_REF: _DataType.ValueType  # 103
    DT_UINT8_REF: _DataType.ValueType  # 104
    DT_INT16_REF: _DataType.ValueType  # 105
    DT_INT8_REF: _DataType.ValueType  # 106
    DT_STRING_REF: _DataType.ValueType  # 107
    DT_COMPLEX64_REF: _DataType.ValueType  # 108
    DT_INT64_REF: _DataType.ValueType  # 109
    DT_BOOL_REF: _DataType.ValueType  # 110
    DT_QINT8_REF: _DataType.ValueType  # 111
    DT_QUINT8_REF: _DataType.ValueType  # 112
    DT_QINT32_REF: _DataType.ValueType  # 113
    DT_BFLOAT16_REF: _DataType.ValueType  # 114
    DT_QINT16_REF: _DataType.ValueType  # 115
    DT_QUINT16_REF: _DataType.ValueType  # 116
    DT_UINT16_REF: _DataType.ValueType  # 117
    DT_COMPLEX128_REF: _DataType.ValueType  # 118
    DT_HALF_REF: _DataType.ValueType  # 119
    DT_RESOURCE_REF: _DataType.ValueType  # 120
    DT_VARIANT_REF: _DataType.ValueType  # 121
    DT_UINT32_REF: _DataType.ValueType  # 122
    DT_UINT64_REF: _DataType.ValueType  # 123
    DT_FLOAT8_E5M2_REF: _DataType.ValueType  # 124
    DT_FLOAT8_E4M3FN_REF: _DataType.ValueType  # 125
    DT_INT4_REF: _DataType.ValueType  # 129
    """TODO - b/299182407: Leaving room for remaining float8 types.
    DT_FLOAT8_E4M3FNUZ_REF = 126;
    DT_FLOAT8_E4M3B11FNUZ_REF = 127;
    DT_FLOAT8_E5M2FNUZ_REF = 128;
    """
    DT_UINT4_REF: _DataType.ValueType  # 130

class DataType(_DataType, metaclass=_DataTypeEnumTypeWrapper):
    """(== suppress_warning documentation-presence ==)
    LINT.IfChange
    """

DT_INVALID: DataType.ValueType  # 0
"""Not a legal value for DataType.  Used to indicate a DataType field
has not been set.
"""
DT_FLOAT: DataType.ValueType  # 1
"""Data types that all computation devices are expected to be
capable to support.
"""
DT_DOUBLE: DataType.ValueType  # 2
DT_INT32: DataType.ValueType  # 3
DT_UINT8: DataType.ValueType  # 4
DT_INT16: DataType.ValueType  # 5
DT_INT8: DataType.ValueType  # 6
DT_STRING: DataType.ValueType  # 7
DT_COMPLEX64: DataType.ValueType  # 8
"""Single-precision complex"""
DT_INT64: DataType.ValueType  # 9
DT_BOOL: DataType.ValueType  # 10
DT_QINT8: DataType.ValueType  # 11
"""Quantized int8"""
DT_QUINT8: DataType.ValueType  # 12
"""Quantized uint8"""
DT_QINT32: DataType.ValueType  # 13
"""Quantized int32"""
DT_BFLOAT16: DataType.ValueType  # 14
"""Float32 truncated to 16 bits."""
DT_QINT16: DataType.ValueType  # 15
"""Quantized int16"""
DT_QUINT16: DataType.ValueType  # 16
"""Quantized uint16"""
DT_UINT16: DataType.ValueType  # 17
DT_COMPLEX128: DataType.ValueType  # 18
"""Double-precision complex"""
DT_HALF: DataType.ValueType  # 19
DT_RESOURCE: DataType.ValueType  # 20
DT_VARIANT: DataType.ValueType  # 21
"""Arbitrary C++ data types"""
DT_UINT32: DataType.ValueType  # 22
DT_UINT64: DataType.ValueType  # 23
DT_FLOAT8_E5M2: DataType.ValueType  # 24
"""5 exponent bits, 2 mantissa bits."""
DT_FLOAT8_E4M3FN: DataType.ValueType  # 25
"""4 exponent bits, 3 mantissa bits, finite-only, with"""
DT_INT4: DataType.ValueType  # 29
"""2 NaNs (0bS1111111).
TODO - b/299182407: Leaving room for remaining float8 types.
DT_FLOAT8_E4M3FNUZ = 26;
DT_FLOAT8_E4M3B11FNUZ = 27;
DT_FLOAT8_E5M2FNUZ = 28;
"""
DT_UINT4: DataType.ValueType  # 30
DT_FLOAT_REF: DataType.ValueType  # 101
"""Do not use!  These are only for TF1's obsolete reference Variables.
Every enum above should have a corresponding value below (verified by
types_test).
"""
DT_DOUBLE_REF: DataType.ValueType  # 102
DT_INT32_REF: DataType.ValueType  # 103
DT_UINT8_REF: DataType.ValueType  # 104
DT_INT16_REF: DataType.ValueType  # 105
DT_INT8_REF: DataType.ValueType  # 106
DT_STRING_REF: DataType.ValueType  # 107
DT_COMPLEX64_REF: DataType.ValueType  # 108
DT_INT64_REF: DataType.ValueType  # 109
DT_BOOL_REF: DataType.ValueType  # 110
DT_QINT8_REF: DataType.ValueType  # 111
DT_QUINT8_REF: DataType.ValueType  # 112
DT_QINT32_REF: DataType.ValueType  # 113
DT_BFLOAT16_REF: DataType.ValueType  # 114
DT_QINT16_REF: DataType.ValueType  # 115
DT_QUINT16_REF: DataType.ValueType  # 116
DT_UINT16_REF: DataType.ValueType  # 117
DT_COMPLEX128_REF: DataType.ValueType  # 118
DT_HALF_REF: DataType.ValueType  # 119
DT_RESOURCE_REF: DataType.ValueType  # 120
DT_VARIANT_REF: DataType.ValueType  # 121
DT_UINT32_REF: DataType.ValueType  # 122
DT_UINT64_REF: DataType.ValueType  # 123
DT_FLOAT8_E5M2_REF: DataType.ValueType  # 124
DT_FLOAT8_E4M3FN_REF: DataType.ValueType  # 125
DT_INT4_REF: DataType.ValueType  # 129
"""TODO - b/299182407: Leaving room for remaining float8 types.
DT_FLOAT8_E4M3FNUZ_REF = 126;
DT_FLOAT8_E4M3B11FNUZ_REF = 127;
DT_FLOAT8_E5M2FNUZ_REF = 128;
"""
DT_UINT4_REF: DataType.ValueType  # 130
global___DataType = DataType

@typing.final
class SerializedDType(google.protobuf.message.Message):
    """Represents a serialized tf.dtypes.Dtype"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATATYPE_FIELD_NUMBER: builtins.int
    datatype: global___DataType.ValueType
    def __init__(self, *, datatype: global___DataType.ValueType | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["datatype", b"datatype"]) -> None: ...

global___SerializedDType = SerializedDType



from _typeshed import Incomplete

class JWTBearerTokenGenerator:
    DEFAULT_EXPIRES_IN: int
    secret_key: Incomplete
    issuer: Incomplete
    alg: Incomplete
    def __init__(self, secret_key, issuer=None, alg: str = "RS256") -> None: ...
    @staticmethod
    def get_allowed_scope(client, scope): ...
    @staticmethod
    def get_sub_value(user): ...
    def get_token_data(self, grant_type, client, expires_in, user=None, scope=None): ...
    def generate(self, grant_type, client, user=None, scope=None, expires_in=None): ...
    def __call__(self, grant_type, client, user=None, scope=None, expires_in=None, include_refresh_token: bool = True): ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
LINT: LEGACY_NAMES"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.wrappers_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _DataType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DataTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DataType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    kFloat: _DataType.ValueType  # 0
    kDouble: _DataType.ValueType  # 1
    kHalf: _DataType.ValueType  # 2
    kInt8: _DataType.ValueType  # 3
    kInt32: _DataType.ValueType  # 4
    kComplexFloat: _DataType.ValueType  # 5
    kComplexDouble: _DataType.ValueType  # 6
    kBF16: _DataType.ValueType  # 7
    kF8E5M2: _DataType.ValueType  # 8
    kF8E4M3FN: _DataType.ValueType  # 9
    kF8E5M2FNUZ: _DataType.ValueType  # 10
    kF8E4M3FNUZ: _DataType.ValueType  # 11
    kInt64: _DataType.ValueType  # 12

class DataType(_DataType, metaclass=_DataTypeEnumTypeWrapper):
    """Specifies the data type used by an operation."""

kFloat: DataType.ValueType  # 0
kDouble: DataType.ValueType  # 1
kHalf: DataType.ValueType  # 2
kInt8: DataType.ValueType  # 3
kInt32: DataType.ValueType  # 4
kComplexFloat: DataType.ValueType  # 5
kComplexDouble: DataType.ValueType  # 6
kBF16: DataType.ValueType  # 7
kF8E5M2: DataType.ValueType  # 8
kF8E4M3FN: DataType.ValueType  # 9
kF8E5M2FNUZ: DataType.ValueType  # 10
kF8E4M3FNUZ: DataType.ValueType  # 11
kInt64: DataType.ValueType  # 12
global___DataType = DataType

class _DataLayout:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DataLayoutEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DataLayout.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    kYXDepthBatch: _DataLayout.ValueType  # 0
    """Naming convention:
    Y <-> row or height
    X <-> column or width
    Batch <-> batch, or N
    Depth <-> feature, or channel
    TODO(timshen): turn them into cuDNN names, e.g. kNCHW.

    Note: In cudnn, kBatchDepthYX4 and kBatchDepthYX32 are the same layout
    (namely, NCHW_VECT_C).  It differentiates between these two by using a
    different data type (int8x4 vs int8x32).  In StreamExecutor we use
    different layouts for these, because we don't usually pass an explicit data
    type to StreamExecutor functions.
    """
    kYXBatchDepth: _DataLayout.ValueType  # 1
    kBatchYXDepth: _DataLayout.ValueType  # 2
    """cuDNN's NHWC layout"""
    kBatchDepthYX: _DataLayout.ValueType  # 3
    """cuDNN's NCHW layout"""
    kBatchDepthYX4: _DataLayout.ValueType  # 4
    """cuDNN's NCHW_VECT_C with 4-elem vectors (e.g. int8x4)"""
    kBatchDepthYX32: _DataLayout.ValueType  # 5
    """cuDNN's NCHW_VECT_C with 32-elem vects (e.g. int8x32)"""

class DataLayout(_DataLayout, metaclass=_DataLayoutEnumTypeWrapper):
    """Describes how a convolution input or output layer's data is formatted."""

kYXDepthBatch: DataLayout.ValueType  # 0
"""Naming convention:
Y <-> row or height
X <-> column or width
Batch <-> batch, or N
Depth <-> feature, or channel
TODO(timshen): turn them into cuDNN names, e.g. kNCHW.

Note: In cudnn, kBatchDepthYX4 and kBatchDepthYX32 are the same layout
(namely, NCHW_VECT_C).  It differentiates between these two by using a
different data type (int8x4 vs int8x32).  In StreamExecutor we use
different layouts for these, because we don't usually pass an explicit data
type to StreamExecutor functions.
"""
kYXBatchDepth: DataLayout.ValueType  # 1
kBatchYXDepth: DataLayout.ValueType  # 2
"""cuDNN's NHWC layout"""
kBatchDepthYX: DataLayout.ValueType  # 3
"""cuDNN's NCHW layout"""
kBatchDepthYX4: DataLayout.ValueType  # 4
"""cuDNN's NCHW_VECT_C with 4-elem vectors (e.g. int8x4)"""
kBatchDepthYX32: DataLayout.ValueType  # 5
"""cuDNN's NCHW_VECT_C with 32-elem vects (e.g. int8x32)"""
global___DataLayout = DataLayout

class _FilterLayout:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _FilterLayoutEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_FilterLayout.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    kOutputInputYX: _FilterLayout.ValueType  # 0
    """Naming convention:
    Y <-> row or height
    X <-> column or width
    Output <-> output feature, or N
    Input <-> input feature, or N
    TODO(timshen): turn them into cuDNN names, e.g. kNCHW.
    cuDNN's NCHW layout
    """
    kOutputYXInput: _FilterLayout.ValueType  # 1
    """cuDNN's NHWC layout"""
    kOutputInputYX4: _FilterLayout.ValueType  # 2
    """cuDNN's NCHW_VECT_C layout with 4-elem vectors"""
    kOutputInputYX32: _FilterLayout.ValueType  # 5
    """cuDNN's NCHW_VECT_C layout with 32-elem vectors"""
    kOutputInputYX32_CudnnReordered: _FilterLayout.ValueType  # 6
    """cuDNN-specific filter reordering (using `cudnnReorderFilterAndBias`)
    When the filter is reordered, so is the bias (if present).
    """
    kInputYXOutput: _FilterLayout.ValueType  # 3
    kYXInputOutput: _FilterLayout.ValueType  # 4

class FilterLayout(_FilterLayout, metaclass=_FilterLayoutEnumTypeWrapper):
    """Describes how a convolution filter is laid out in the memory."""

kOutputInputYX: FilterLayout.ValueType  # 0
"""Naming convention:
Y <-> row or height
X <-> column or width
Output <-> output feature, or N
Input <-> input feature, or N
TODO(timshen): turn them into cuDNN names, e.g. kNCHW.
cuDNN's NCHW layout
"""
kOutputYXInput: FilterLayout.ValueType  # 1
"""cuDNN's NHWC layout"""
kOutputInputYX4: FilterLayout.ValueType  # 2
"""cuDNN's NCHW_VECT_C layout with 4-elem vectors"""
kOutputInputYX32: FilterLayout.ValueType  # 5
"""cuDNN's NCHW_VECT_C layout with 32-elem vectors"""
kOutputInputYX32_CudnnReordered: FilterLayout.ValueType  # 6
"""cuDNN-specific filter reordering (using `cudnnReorderFilterAndBias`)
When the filter is reordered, so is the bias (if present).
"""
kInputYXOutput: FilterLayout.ValueType  # 3
kYXInputOutput: FilterLayout.ValueType  # 4
global___FilterLayout = FilterLayout

class _ActivationMode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ActivationModeEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ActivationMode.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    kNone: _ActivationMode.ValueType  # 0
    kSigmoid: _ActivationMode.ValueType  # 1
    kRelu: _ActivationMode.ValueType  # 2
    """Rectified linear activation: f(x) = x < 0 ? 0 : x"""
    kRelu6: _ActivationMode.ValueType  # 3
    """Rectified linear activation; where upper maximum is 6.0."""
    kReluX: _ActivationMode.ValueType  # 4
    """Rectified linear activation; where upper maximum specified by
    BatchDescriptor::value_max().
    """
    kTanh: _ActivationMode.ValueType  # 5
    kBandPass: _ActivationMode.ValueType  # 6
    """Like ReluX; but passes all values in the range [-X,X]."""
    kElu: _ActivationMode.ValueType  # 7
    """Exponential linear activation: f(x) = x < 0 ? e^x - 1 : x"""
    kLeakyRelu: _ActivationMode.ValueType  # 8
    """Leaky Rectified linear activation: f(x) = x < 0 ? alpha * x : x"""
    kGeluExact: _ActivationMode.ValueType  # 9
    """Gaussian Error linear unit activation:
      x * P(X <= x) = 0.5 * x * (1 + erf(x / sqrt(2))), where P(X) ~ N(0, 1).
    """

class ActivationMode(_ActivationMode, metaclass=_ActivationModeEnumTypeWrapper):
    """Describes a kind of non-linearity (threshold-like mathematical function)."""

kNone: ActivationMode.ValueType  # 0
kSigmoid: ActivationMode.ValueType  # 1
kRelu: ActivationMode.ValueType  # 2
"""Rectified linear activation: f(x) = x < 0 ? 0 : x"""
kRelu6: ActivationMode.ValueType  # 3
"""Rectified linear activation; where upper maximum is 6.0."""
kReluX: ActivationMode.ValueType  # 4
"""Rectified linear activation; where upper maximum specified by
BatchDescriptor::value_max().
"""
kTanh: ActivationMode.ValueType  # 5
kBandPass: ActivationMode.ValueType  # 6
"""Like ReluX; but passes all values in the range [-X,X]."""
kElu: ActivationMode.ValueType  # 7
"""Exponential linear activation: f(x) = x < 0 ? e^x - 1 : x"""
kLeakyRelu: ActivationMode.ValueType  # 8
"""Leaky Rectified linear activation: f(x) = x < 0 ? alpha * x : x"""
kGeluExact: ActivationMode.ValueType  # 9
"""Gaussian Error linear unit activation:
  x * P(X <= x) = 0.5 * x * (1 + erf(x / sqrt(2))), where P(X) ~ N(0, 1).
"""
global___ActivationMode = ActivationMode

class _ConvolutionMode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ConvolutionModeEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ConvolutionMode.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    CROSS_CORRELATION: _ConvolutionMode.ValueType  # 0
    CONVOLUTION: _ConvolutionMode.ValueType  # 1

class ConvolutionMode(_ConvolutionMode, metaclass=_ConvolutionModeEnumTypeWrapper):
    """Describe the math definition for the conv op. The popular behavior is
    actually called cross-correlation in math, despite the operation is often
    referred as convolution. See cuDNN cudnnConvolutionMode_t.
    """

CROSS_CORRELATION: ConvolutionMode.ValueType  # 0
CONVOLUTION: ConvolutionMode.ValueType  # 1
global___ConvolutionMode = ConvolutionMode

class _ConvolutionKind:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ConvolutionKindEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ConvolutionKind.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID: _ConvolutionKind.ValueType  # 0
    FORWARD: _ConvolutionKind.ValueType  # 1
    BACKWARD_FILTER: _ConvolutionKind.ValueType  # 2
    BACKWARD_DATA: _ConvolutionKind.ValueType  # 3
    FORWARD_BIAS_ACTIVATION: _ConvolutionKind.ValueType  # 4
    FORWARD_GRAPH: _ConvolutionKind.ValueType  # 5

class ConvolutionKind(_ConvolutionKind, metaclass=_ConvolutionKindEnumTypeWrapper): ...

INVALID: ConvolutionKind.ValueType  # 0
FORWARD: ConvolutionKind.ValueType  # 1
BACKWARD_FILTER: ConvolutionKind.ValueType  # 2
BACKWARD_DATA: ConvolutionKind.ValueType  # 3
FORWARD_BIAS_ACTIVATION: ConvolutionKind.ValueType  # 4
FORWARD_GRAPH: ConvolutionKind.ValueType  # 5
global___ConvolutionKind = ConvolutionKind

class _NormKind:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _NormKindEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_NormKind.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    LAYER_FWD_INFER: _NormKind.ValueType  # 0
    LAYER_FWD_TRAIN: _NormKind.ValueType  # 1
    LAYER_BWD: _NormKind.ValueType  # 2

class NormKind(_NormKind, metaclass=_NormKindEnumTypeWrapper):
    """NormKind kind"""

LAYER_FWD_INFER: NormKind.ValueType  # 0
LAYER_FWD_TRAIN: NormKind.ValueType  # 1
LAYER_BWD: NormKind.ValueType  # 2
global___NormKind = NormKind

class _FusedMHAKind:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _FusedMHAKindEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_FusedMHAKind.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    BMM1_OUTPUT_UNKNOWN: _FusedMHAKind.ValueType  # 0
    BMM1_OUTPUT_INPUT_TYPE: _FusedMHAKind.ValueType  # 1
    BMM1_OUTPUT_FLOAT: _FusedMHAKind.ValueType  # 2

class FusedMHAKind(_FusedMHAKind, metaclass=_FusedMHAKindEnumTypeWrapper):
    """FusedMHAKind kind"""

BMM1_OUTPUT_UNKNOWN: FusedMHAKind.ValueType  # 0
BMM1_OUTPUT_INPUT_TYPE: FusedMHAKind.ValueType  # 1
BMM1_OUTPUT_FLOAT: FusedMHAKind.ValueType  # 2
global___FusedMHAKind = FusedMHAKind

class _FMHAMaskKind:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _FMHAMaskKindEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_FMHAMaskKind.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    NO_MASK: _FMHAMaskKind.ValueType  # 0
    PADDING: _FMHAMaskKind.ValueType  # 1
    CAUSAL: _FMHAMaskKind.ValueType  # 2
    PADDING_CAUSAL: _FMHAMaskKind.ValueType  # 3
    ALIBI: _FMHAMaskKind.ValueType  # 4

class FMHAMaskKind(_FMHAMaskKind, metaclass=_FMHAMaskKindEnumTypeWrapper):
    """FusedMHAMaskKind kind"""

NO_MASK: FMHAMaskKind.ValueType  # 0
PADDING: FMHAMaskKind.ValueType  # 1
CAUSAL: FMHAMaskKind.ValueType  # 2
PADDING_CAUSAL: FMHAMaskKind.ValueType  # 3
ALIBI: FMHAMaskKind.ValueType  # 4
global___FMHAMaskKind = FMHAMaskKind

@typing.final
class TensorDescriptorProto(google.protobuf.message.Message):
    """Generic tensor representation."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DIMENSIONS_FIELD_NUMBER: builtins.int
    DATA_TYPE_FIELD_NUMBER: builtins.int
    DATA_LAYOUT_FIELD_NUMBER: builtins.int
    FILTER_LAYOUT_FIELD_NUMBER: builtins.int
    data_type: global___DataType.ValueType
    data_layout: global___DataLayout.ValueType
    filter_layout: global___FilterLayout.ValueType
    @property
    def dimensions(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        dimensions: collections.abc.Iterable[builtins.int] | None = ...,
        data_type: global___DataType.ValueType | None = ...,
        data_layout: global___DataLayout.ValueType | None = ...,
        filter_layout: global___FilterLayout.ValueType | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "data_layout", b"data_layout", "filter_layout", b"filter_layout", "layout_oneof", b"layout_oneof"
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "data_layout",
            b"data_layout",
            "data_type",
            b"data_type",
            "dimensions",
            b"dimensions",
            "filter_layout",
            b"filter_layout",
            "layout_oneof",
            b"layout_oneof",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["layout_oneof", b"layout_oneof"]
    ) -> typing.Literal["data_layout", "filter_layout"] | None: ...

global___TensorDescriptorProto = TensorDescriptorProto

@typing.final
class AlgorithmProto(google.protobuf.message.Message):
    """Generic algorithm representation."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _MathType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _MathTypeEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[AlgorithmProto._MathType.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        DEFAULT_MATH: AlgorithmProto._MathType.ValueType  # 0
        TENSOR_OP_MATH: AlgorithmProto._MathType.ValueType  # 1
        """The GPU may operate 4x4 matrix FMA.
        See cuDNN's documentation for CUDNN_TENSOR_OP_MATH.
        """

    class MathType(_MathType, metaclass=_MathTypeEnumTypeWrapper): ...
    DEFAULT_MATH: AlgorithmProto.MathType.ValueType  # 0
    TENSOR_OP_MATH: AlgorithmProto.MathType.ValueType  # 1
    """The GPU may operate 4x4 matrix FMA.
    See cuDNN's documentation for CUDNN_TENSOR_OP_MATH.
    """

    @typing.final
    class TuningKnobsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        value: builtins.int
        def __init__(self, *, key: builtins.int | None = ..., value: builtins.int | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    ALGO_ID_FIELD_NUMBER: builtins.int
    MATH_TYPE_FIELD_NUMBER: builtins.int
    TUNING_KNOBS_FIELD_NUMBER: builtins.int
    IS_CUDNN_FRONTEND_FIELD_NUMBER: builtins.int
    WORKSPACE_SIZE_FIELD_NUMBER: builtins.int
    algo_id: builtins.int
    math_type: global___AlgorithmProto.MathType.ValueType
    is_cudnn_frontend: builtins.bool
    """Legacy algorithm enums and cuDNN Frontend engine numbers need to coexist in
    the same proto medium-term, until we can be confident of no longer needing
    the legacy cuDNN convolution API.  Once the migration is complete, we can
    stop producing legacy algorithm enums and remove this field.
    """
    @property
    def tuning_knobs(self) -> google.protobuf.internal.containers.ScalarMap[builtins.int, builtins.int]: ...
    @property
    def workspace_size(self) -> google.protobuf.wrappers_pb2.UInt64Value:
        """For ROCm only, it's impossible to re-query the required workspace size
        after running the algorithm search, so we must store the workspace size
        along with the choice of algorithm.  For consistency and convenience,
        cuDNN uses this field in the same way, even though it would be possible to
        re-query the workspace size from cuDNN at each use.

        Since this message is persisted in files, we need to be able to distinguish
        0 workspace size from unknown workspace size in an old message, so this is
        a message field.
        """

    def __init__(
        self,
        *,
        algo_id: builtins.int | None = ...,
        math_type: global___AlgorithmProto.MathType.ValueType | None = ...,
        tuning_knobs: collections.abc.Mapping[builtins.int, builtins.int] | None = ...,
        is_cudnn_frontend: builtins.bool | None = ...,
        workspace_size: google.protobuf.wrappers_pb2.UInt64Value | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["workspace_size", b"workspace_size"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "algo_id",
            b"algo_id",
            "is_cudnn_frontend",
            b"is_cudnn_frontend",
            "math_type",
            b"math_type",
            "tuning_knobs",
            b"tuning_knobs",
            "workspace_size",
            b"workspace_size",
        ],
    ) -> None: ...

global___AlgorithmProto = AlgorithmProto

@typing.final
class AlgorithmConfigProto(google.protobuf.message.Message):
    """Proto definition of AlgorithmConfig in "dnn.h".
    TODO(ruochengw): After cl/380702564 is submitted, add support for algorithm
    configs with cuDNN Frontend APIs.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ALGORITHM_FIELD_NUMBER: builtins.int
    ALGORITHM_NO_SCRATCH_FIELD_NUMBER: builtins.int
    SCRATCH_SIZE_FIELD_NUMBER: builtins.int
    scratch_size: builtins.int
    @property
    def algorithm(self) -> global___AlgorithmProto: ...
    @property
    def algorithm_no_scratch(self) -> global___AlgorithmProto: ...
    def __init__(
        self,
        *,
        algorithm: global___AlgorithmProto | None = ...,
        algorithm_no_scratch: global___AlgorithmProto | None = ...,
        scratch_size: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "algorithm",
            b"algorithm",
            "algorithm_no_scratch",
            b"algorithm_no_scratch",
            "optional_algorithm",
            b"optional_algorithm",
            "optional_algorithm_no_scratch",
            b"optional_algorithm_no_scratch",
            "optional_scratch_size",
            b"optional_scratch_size",
            "scratch_size",
            b"scratch_size",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "algorithm",
            b"algorithm",
            "algorithm_no_scratch",
            b"algorithm_no_scratch",
            "optional_algorithm",
            b"optional_algorithm",
            "optional_algorithm_no_scratch",
            b"optional_algorithm_no_scratch",
            "optional_scratch_size",
            b"optional_scratch_size",
            "scratch_size",
            b"scratch_size",
        ],
    ) -> None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_algorithm", b"optional_algorithm"]
    ) -> typing.Literal["algorithm"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_algorithm_no_scratch", b"optional_algorithm_no_scratch"]
    ) -> typing.Literal["algorithm_no_scratch"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_scratch_size", b"optional_scratch_size"]
    ) -> typing.Literal["scratch_size"] | None: ...

global___AlgorithmConfigProto = AlgorithmConfigProto

@typing.final
class ConvolutionDescriptorProto(google.protobuf.message.Message):
    """Convolution-specific parameters."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PADDINGS_FIELD_NUMBER: builtins.int
    STRIDES_FIELD_NUMBER: builtins.int
    DILATIONS_FIELD_NUMBER: builtins.int
    COMPUTE_MODE_FIELD_NUMBER: builtins.int
    GROUP_COUNT_FIELD_NUMBER: builtins.int
    CONVOLUTION_MODE_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    compute_mode: global___DataType.ValueType
    """The "accumulator" type. For example, use F32 as an accumulator for F16
    convolutions.
    See cuDNN's cudnnConvolutionMode_t.
    """
    group_count: builtins.int
    """See cuDNN's group count."""
    convolution_mode: global___ConvolutionMode.ValueType
    name: builtins.str
    """Tensorflow node name, same as in NodeDef, for debugging purposes."""
    @property
    def paddings(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def strides(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def dilations(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        paddings: collections.abc.Iterable[builtins.int] | None = ...,
        strides: collections.abc.Iterable[builtins.int] | None = ...,
        dilations: collections.abc.Iterable[builtins.int] | None = ...,
        compute_mode: global___DataType.ValueType | None = ...,
        group_count: builtins.int | None = ...,
        convolution_mode: global___ConvolutionMode.ValueType | None = ...,
        name: builtins.str | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "compute_mode",
            b"compute_mode",
            "convolution_mode",
            b"convolution_mode",
            "dilations",
            b"dilations",
            "group_count",
            b"group_count",
            "name",
            b"name",
            "paddings",
            b"paddings",
            "strides",
            b"strides",
        ],
    ) -> None: ...

global___ConvolutionDescriptorProto = ConvolutionDescriptorProto



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["harmonic_function", "local_and_global_consistency"]

@_dispatchable
def harmonic_function(G: Graph[_Node], max_iter: int = 30, label_name: str = "label"): ...
@_dispatchable
def local_and_global_consistency(G: Graph[_Node], alpha: float = 0.99, max_iter: int = 30, label_name: str = "label"): ...



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["current_flow_betweenness_centrality_subset", "edge_current_flow_betweenness_centrality_subset"]

@_dispatchable
def current_flow_betweenness_centrality_subset(
    G: Graph[_Node],
    sources: Iterable[_Node],
    targets: Iterable[_Node],
    normalized: bool | None = True,
    weight: str | None = None,
    dtype: type = ...,
    solver: str = "lu",
) -> dict[Incomplete, float]: ...
@_dispatchable
def edge_current_flow_betweenness_centrality_subset(
    G: Graph[_Node],
    sources: Iterable[_Node],
    targets: Iterable[_Node],
    normalized: bool | None = True,
    weight: str | None = None,
    dtype: type = ...,
    solver: str = "lu",
) -> dict[tuple[Incomplete, Incomplete], float]: ...



#!/usr/bin/env python3
"""Run mypy on typeshed's stdlib and third-party stubs."""

from __future__ import annotations

import argparse
import concurrent.futures
import os
import subprocess
import sys
import tempfile
import time
from collections import defaultdict
from dataclasses import dataclass
from enum import Enum
from itertools import product
from pathlib import Path
from threading import Lock
from typing import Annotated, Any, NamedTuple
from typing_extensions import TypeAlias

from packaging.requirements import Requirement

from ts_utils.metadata import PackageDependencies, get_recursive_requirements, read_metadata
from ts_utils.mypy import MypyDistConf, mypy_configuration_from_distribution, temporary_mypy_config_file
from ts_utils.paths import STDLIB_PATH, STUBS_PATH, TESTS_DIR, TS_BASE_PATH, distribution_path
from ts_utils.utils import (
    PYTHON_VERSION,
    colored,
    get_gitignore_spec,
    get_mypy_req,
    parse_stdlib_versions_file,
    print_error,
    print_success_msg,
    spec_matches_path,
    supported_versions_for_module,
    venv_python,
)

# Fail early if mypy isn't installed
try:
    import mypy  # pyright: ignore[reportUnusedImport]  # noqa: F401
except ImportError:
    print_error("Cannot import mypy. Did you install it?")
    sys.exit(1)

SUPPORTED_VERSIONS = ["3.14", "3.13", "3.12", "3.11", "3.10", "3.9"]
SUPPORTED_PLATFORMS = ("linux", "win32", "darwin")
DIRECTORIES_TO_TEST = [STDLIB_PATH, STUBS_PATH]

VersionString: TypeAlias = Annotated[str, "Must be one of the entries in SUPPORTED_VERSIONS"]
Platform: TypeAlias = Annotated[str, "Must be one of the entries in SUPPORTED_PLATFORMS"]


@dataclass(init=False)
class CommandLineArgs:
    verbose: int
    filter: list[Path]
    exclude: list[Path] | None
    python_version: list[VersionString] | None
    platform: list[Platform] | None


def valid_path(cmd_arg: str) -> Path:
    """Parse a CLI argument that is intended to point to a valid typeshed path."""
    path = Path(cmd_arg)
    if not path.exists():
        raise argparse.ArgumentTypeError(f'"{path}" does not exist in typeshed!')
    if not (path in DIRECTORIES_TO_TEST or any(directory in path.parents for directory in DIRECTORIES_TO_TEST)):
        raise argparse.ArgumentTypeError('mypy_test.py only tests the stubs found in the "stdlib" and "stubs" directories')
    return path


def remove_dev_suffix(version: str) -> str:
    """Remove the `-dev` suffix from a version string.

    This is a helper function for argument-parsing.
    """
    if version.endswith("-dev"):
        return version[: -len("-dev")]
    return ".".join(version.split(".")[:2])


parser = argparse.ArgumentParser(
    description="Typecheck typeshed's stubs with mypy. Patterns are unanchored regexps on the full path."
)
parser.add_argument(
    "filter",
    type=valid_path,
    nargs="*",
    help='Test these files and directories (defaults to all files in the "stdlib" and "stubs" directories)',
)
parser.add_argument("-x", "--exclude", type=valid_path, nargs="*", help="Exclude these files and directories")
parser.add_argument("-v", "--verbose", action="count", default=0, help="More output")
parser.add_argument(
    "-p",
    "--python-version",
    type=remove_dev_suffix,
    choices=SUPPORTED_VERSIONS,
    nargs="*",
    action="extend",
    help="These versions only (major[.minor])",
)
parser.add_argument(
    "--platform",
    choices=SUPPORTED_PLATFORMS,
    nargs="*",
    action="extend",
    help="Run mypy for certain OS platforms (defaults to sys.platform only)",
)


@dataclass
class TestConfig:
    """Configuration settings for a single run of the `test_typeshed` function."""

    verbose: int
    filter: list[Path]
    exclude: list[Path]
    version: VersionString
    platform: Platform


def log(args: TestConfig, *varargs: object) -> None:
    if args.verbose >= 2:
        print(colored(" ".join(map(str, varargs)), "blue"))


def match(path: Path, args: TestConfig) -> bool:
    for excluded_path in args.exclude:
        if path == excluded_path:
            log(args, path, "explicitly excluded")
            return False
        if excluded_path in path.parents:
            log(args, path, f'is in an explicitly excluded directory "{excluded_path}"')
            return False
    for included_path in args.filter:
        if path == included_path:
            log(args, path, "was explicitly included")
            return True
        if included_path in path.parents:
            log(args, path, f'is in an explicitly included directory "{included_path}"')
            return True
    log_msg = (
        f'is implicitly excluded: was not in any of the directories or paths specified on the command line: "{args.filter!r}"'
    )
    log(args, path, log_msg)
    return False


def add_files(files: list[Path], module: Path, args: TestConfig) -> None:
    """Add all files in package or module represented by 'name' located in 'root'."""
    if module.name.startswith("."):
        return
    if module.is_file() and module.suffix == ".pyi":
        if match(module, args):
            files.append(module)
    else:
        files.extend(sorted(file for file in module.rglob("*.pyi") if match(file, args)))


class MypyResult(Enum):
    SUCCESS = 0
    FAILURE = 1
    CRASH = 2

    @staticmethod
    def from_process_result(result: subprocess.CompletedProcess[Any]) -> MypyResult:
        if result.returncode == 0:
            return MypyResult.SUCCESS
        elif result.returncode == 1:
            return MypyResult.FAILURE
        else:
            return MypyResult.CRASH


def run_mypy(
    args: TestConfig,
    configurations: list[MypyDistConf],
    files: list[Path],
    *,
    testing_stdlib: bool,
    non_types_dependencies: bool,
    venv_dir: Path | None,
    mypypath: str | None = None,
) -> MypyResult:
    env_vars = dict(os.environ)
    if mypypath is not None:
        env_vars["MYPYPATH"] = mypypath
    with temporary_mypy_config_file(configurations) as temp:
        flags = [
            "--python-version",
            args.version,
            "--show-traceback",
            "--warn-incomplete-stub",
            "--no-error-summary",
            "--platform",
            args.platform,
            "--custom-typeshed-dir",
            str(TS_BASE_PATH),
            "--strict",
            # Stub completion is checked by pyright (--allow-*-defs)
            "--allow-untyped-defs",
            "--allow-incomplete-defs",
            # See https://github.com/python/typeshed/pull/9491#issuecomment-1381574946
            # for discussion and reasoning to keep "--allow-subclassing-any"
            "--allow-subclassing-any",
            "--enable-error-code",
            "ignore-without-code",
            "--enable-error-code",
            "redundant-self",
            "--config-file",
            temp.name,
        ]
        if not testing_stdlib:
            flags.append("--explicit-package-bases")
        if not non_types_dependencies:
            flags.append("--no-site-packages")

        mypy_args = [*flags, *map(str, files)]
        python_path = sys.executable if venv_dir is None else str(venv_python(venv_dir))
        mypy_command = [python_path, "-m", "mypy", *mypy_args]
        if args.verbose:
            print(colored(f"running {' '.join(mypy_command)}", "blue"))
        result = subprocess.run(mypy_command, capture_output=True, text=True, env=env_vars, check=False)
    if result.returncode:
        print_error(f"failure (exit code {result.returncode})\n")
        if result.stdout:
            print_error(result.stdout)
        if result.stderr:
            print_error(result.stderr)
        if non_types_dependencies and args.verbose:
            print("Ran with the following environment:")
            subprocess.run(["uv", "pip", "freeze"], env={**os.environ, "VIRTUAL_ENV": str(venv_dir)}, check=False)
            print()
    else:
        print_success_msg()

    return MypyResult.from_process_result(result)


def add_third_party_files(distribution: str, files: list[Path], args: TestConfig, seen_dists: set[str]) -> None:
    typeshed_reqs = get_recursive_requirements(distribution).typeshed_pkgs
    if distribution in seen_dists:
        return
    seen_dists.add(distribution)
    seen_dists.update(r.name for r in typeshed_reqs)
    root = distribution_path(distribution)
    for path in root.iterdir():
        add_files(files, path, args)


class TestResult(NamedTuple):
    mypy_result: MypyResult
    files_checked: int


def test_third_party_distribution(
    distribution: str, args: TestConfig, venv_dir: Path | None, *, non_types_dependencies: bool
) -> TestResult:
    """Test the stubs of a third-party distribution.

    Return a tuple, where the first element indicates mypy's return code
    and the second element is the number of checked files.
    """
    files: list[Path] = []
    seen_dists: set[str] = set()
    add_third_party_files(distribution, files, args, seen_dists)
    configurations = mypy_configuration_from_distribution(distribution)

    if not files and args.filter:
        return TestResult(MypyResult.SUCCESS, 0)

    print(f"testing {distribution} ({len(files)} files)... ", end="", flush=True)

    if not files:
        print_error("no files found")
        sys.exit(1)

    mypypath = os.pathsep.join(str(distribution_path(dist)) for dist in seen_dists)
    if args.verbose:
        print(colored(f"\nMYPYPATH={mypypath}", "blue"))
    result = run_mypy(
        args,
        configurations,
        files,
        venv_dir=venv_dir,
        mypypath=mypypath,
        testing_stdlib=False,
        non_types_dependencies=non_types_dependencies,
    )
    return TestResult(result, len(files))


def test_stdlib(args: TestConfig) -> TestResult:
    files: list[Path] = []
    for file in STDLIB_PATH.iterdir():
        if file.name in ("VERSIONS", TESTS_DIR):
            continue
        add_files(files, file, args)

    files = remove_modules_not_in_python_version(files, args.version)

    if not files:
        return TestResult(MypyResult.SUCCESS, 0)

    print(f"Testing stdlib ({len(files)} files)... ", end="", flush=True)
    # We don't actually need to install anything for the stdlib testing
    result = run_mypy(args, [], files, venv_dir=None, testing_stdlib=True, non_types_dependencies=False)
    return TestResult(result, len(files))


def remove_modules_not_in_python_version(paths: list[Path], py_version: VersionString) -> list[Path]:
    py_version_tuple = tuple(map(int, py_version.split(".")))
    module_versions = parse_stdlib_versions_file()
    new_paths: list[Path] = []
    for path in paths:
        if path.parts[0] != "stdlib" or path.suffix != ".pyi":
            continue
        module_name = stdlib_module_name_from_path(path)
        min_version, max_version = supported_versions_for_module(module_versions, module_name)
        if min_version <= py_version_tuple <= max_version:
            new_paths.append(path)
    return new_paths


def stdlib_module_name_from_path(path: Path) -> str:
    assert path.parts[0] == "stdlib"
    assert path.suffix == ".pyi"
    parts = list(path.parts[1:-1])
    if path.parts[-1] != "__init__.pyi":
        parts.append(path.parts[-1].removesuffix(".pyi"))
    return ".".join(parts)


@dataclass
class TestSummary:
    mypy_result: MypyResult = MypyResult.SUCCESS
    files_checked: int = 0
    packages_skipped: int = 0
    packages_with_errors: int = 0

    def register_result(self, mypy_result: MypyResult, files_checked: int) -> None:
        if mypy_result.value > self.mypy_result.value:
            self.mypy_result = mypy_result
        if mypy_result != MypyResult.SUCCESS:
            self.packages_with_errors += 1
        self.files_checked += files_checked

    def skip_package(self) -> None:
        self.packages_skipped += 1

    def merge(self, other: TestSummary) -> None:
        if other.mypy_result.value > self.mypy_result.value:
            self.mypy_result = other.mypy_result
        self.files_checked += other.files_checked
        self.packages_skipped += other.packages_skipped
        self.packages_with_errors += other.packages_with_errors


_PRINT_LOCK = Lock()
_DISTRIBUTION_TO_VENV_MAPPING: dict[str, Path | None] = {}


def setup_venv_for_external_requirements_set(
    requirements_set: frozenset[Requirement], tempdir: Path, args: TestConfig
) -> tuple[frozenset[Requirement], Path]:
    venv_dir = tempdir / f".venv-{hash(requirements_set)}"
    uv_command = ["uv", "venv", str(venv_dir)]
    if not args.verbose:
        uv_command.append("--quiet")
    subprocess.run(uv_command, check=True)
    return requirements_set, venv_dir


def install_requirements_for_venv(venv_dir: Path, args: TestConfig, external_requirements: frozenset[Requirement]) -> None:
    req_args = sorted(str(req) for req in external_requirements)
    # Use --no-cache-dir to avoid issues with concurrent read/writes to the cache
    uv_command = ["uv", "pip", "install", get_mypy_req(), *req_args, "--no-cache-dir"]
    if args.verbose:
        with _PRINT_LOCK:
            print(colored(f"Running {uv_command}", "blue"))
    else:
        uv_command.append("--quiet")
    try:
        subprocess.run(uv_command, check=True, text=True, env={**os.environ, "VIRTUAL_ENV": str(venv_dir)})
    except subprocess.CalledProcessError as e:
        print(e.stderr)
        raise


def setup_virtual_environments(distributions: dict[str, PackageDependencies], args: TestConfig, tempdir: Path) -> None:
    """Logic necessary for testing stubs with non-types dependencies in isolated environments."""
    if not distributions:
        return  # hooray! Nothing to do

    # STAGE 1: Determine which (if any) stubs packages require virtual environments.
    # Group stubs packages according to their external-requirements sets
    external_requirements_to_distributions: defaultdict[frozenset[Requirement], list[str]] = defaultdict(list)
    num_pkgs_with_external_reqs = 0

    for distribution_name, requirements in distributions.items():
        if requirements.external_pkgs:
            num_pkgs_with_external_reqs += 1
            external_requirements = frozenset(requirements.external_pkgs)
            external_requirements_to_distributions[external_requirements].append(distribution_name)
        else:
            _DISTRIBUTION_TO_VENV_MAPPING[distribution_name] = None

    # Exit early if there are no stubs packages that have non-types dependencies
    if num_pkgs_with_external_reqs == 0:
        if args.verbose:
            print(colored("No additional venvs are required to be set up", "blue"))
        return

    # STAGE 2: Setup a virtual environment for each unique set of external requirements
    requirements_sets_to_venvs: dict[frozenset[Requirement], Path] = {}

    if args.verbose:
        num_venvs = len(external_requirements_to_distributions)
        msg = (
            f"Setting up {num_venvs} venv{'s' if num_venvs != 1 else ''} "
            f"for {num_pkgs_with_external_reqs} "
            f"distribution{'s' if num_pkgs_with_external_reqs != 1 else ''}... "
        )
        print(colored(msg, "blue"), end="", flush=True)

    venv_start_time = time.perf_counter()

    with concurrent.futures.ProcessPoolExecutor() as executor:
        venv_futures = [
            executor.submit(setup_venv_for_external_requirements_set, requirements_set, tempdir, args)
            for requirements_set in external_requirements_to_distributions
        ]
        for venv_future in concurrent.futures.as_completed(venv_futures):
            requirements_set, venv_dir = venv_future.result()
            requirements_sets_to_venvs[requirements_set] = venv_dir

    venv_elapsed_time = time.perf_counter() - venv_start_time

    if args.verbose:
        print(colored(f"took {venv_elapsed_time:.2f} seconds", "blue"))

    # STAGE 3: For each {virtual_environment: requirements_set} pairing,
    # `uv pip install` the requirements set into the virtual environment
    pip_start_time = time.perf_counter()

    # Limit workers to 10 at a time, since this makes network requests
    with concurrent.futures.ThreadPoolExecutor(max_workers=10) as executor:
        pip_install_futures = [
            executor.submit(install_requirements_for_venv, venv_dir, args, requirements_set)
            for requirements_set, venv_dir in requirements_sets_to_venvs.items()
        ]
        concurrent.futures.wait(pip_install_futures)

    pip_elapsed_time = time.perf_counter() - pip_start_time

    if args.verbose:
        msg = f"Combined time for installing requirements across all venvs: {pip_elapsed_time:.2f} seconds"
        print(colored(msg, "blue"))

    # STAGE 4: Populate the _DISTRIBUTION_TO_VENV_MAPPING
    # so that we have a simple {distribution: venv_to_use} mapping to use for the rest of the test.
    for requirements_set, distribution_list in external_requirements_to_distributions.items():
        venv_to_use = requirements_sets_to_venvs[requirements_set]
        _DISTRIBUTION_TO_VENV_MAPPING.update(dict.fromkeys(distribution_list, venv_to_use))


def test_third_party_stubs(args: TestConfig, tempdir: Path) -> TestSummary:
    print("Testing third-party packages...")
    summary = TestSummary()
    gitignore_spec = get_gitignore_spec()
    distributions_to_check: dict[str, PackageDependencies] = {}

    for distribution in sorted(os.listdir("stubs")):
        dist_path = distribution_path(distribution)

        if spec_matches_path(gitignore_spec, dist_path):
            continue

        if dist_path in args.filter or STUBS_PATH in args.filter or any(dist_path in path.parents for path in args.filter):
            metadata = read_metadata(distribution)
            if not metadata.requires_python.contains(PYTHON_VERSION):
                msg = (
                    f"skipping {distribution!r} (requires Python {metadata.requires_python}; "
                    f"test is being run using Python {PYTHON_VERSION})"
                )
                print(colored(msg, "yellow"))
                summary.skip_package()
                continue
            if not metadata.requires_python.contains(args.version):
                msg = f"skipping {distribution!r} for target Python {args.version} (requires Python {metadata.requires_python})"
                print(colored(msg, "yellow"))
                summary.skip_package()
                continue

            distributions_to_check[distribution] = get_recursive_requirements(distribution)

    # Setup the necessary virtual environments for testing the third-party stubs.
    # Note that some stubs may not be tested on all Python versions
    # (due to version incompatibilities),
    # so we can't guarantee that setup_virtual_environments()
    # will only be called once per session.
    distributions_without_venv = {
        distribution: requirements
        for distribution, requirements in distributions_to_check.items()
        if distribution not in _DISTRIBUTION_TO_VENV_MAPPING
    }
    setup_virtual_environments(distributions_without_venv, args, tempdir)

    # Check that there is a venv for every distribution we're testing.
    # Some venvs may exist from previous runs but are skipped in this run.
    assert _DISTRIBUTION_TO_VENV_MAPPING.keys() >= distributions_to_check.keys()

    for distribution in distributions_to_check:
        venv_dir = _DISTRIBUTION_TO_VENV_MAPPING[distribution]
        non_types_dependencies = venv_dir is not None
        mypy_result, files_checked = test_third_party_distribution(
            distribution, args, venv_dir=venv_dir, non_types_dependencies=non_types_dependencies
        )
        summary.register_result(mypy_result, files_checked)

    return summary


def test_typeshed(args: TestConfig, tempdir: Path) -> TestSummary:
    print(f"*** Testing Python {args.version} on {args.platform}")
    summary = TestSummary()

    if STDLIB_PATH in args.filter or any(STDLIB_PATH in path.parents for path in args.filter):
        mypy_result, files_checked = test_stdlib(args)
        summary.register_result(mypy_result, files_checked)
        print()

    if STUBS_PATH in args.filter or any(STUBS_PATH in path.parents for path in args.filter):
        tp_results = test_third_party_stubs(args, tempdir)
        summary.merge(tp_results)
        print()

    return summary


def main() -> None:
    args = parser.parse_args(namespace=CommandLineArgs())
    versions = args.python_version or SUPPORTED_VERSIONS
    platforms = args.platform or [sys.platform]
    path_filter = args.filter or DIRECTORIES_TO_TEST
    exclude = args.exclude or []
    summary = TestSummary()
    with tempfile.TemporaryDirectory() as td:
        td_path = Path(td)
        for version, platform in product(versions, platforms):
            config = TestConfig(args.verbose, path_filter, exclude, version, platform)
            version_summary = test_typeshed(args=config, tempdir=td_path)
            summary.merge(version_summary)

    if summary.mypy_result == MypyResult.FAILURE:
        plural1 = "" if summary.packages_with_errors == 1 else "s"
        plural2 = "" if summary.files_checked == 1 else "s"
        print_error(
            f"--- {summary.packages_with_errors} package{plural1} with errors, {summary.files_checked} file{plural2} checked ---"
        )
        sys.exit(1)
    if summary.mypy_result == MypyResult.CRASH:
        plural = "" if summary.files_checked == 1 else "s"
        print_error(f"--- mypy crashed, {summary.files_checked} file{plural} checked ---")
        sys.exit(2)
    if summary.packages_skipped:
        plural = "" if summary.packages_skipped == 1 else "s"
        print(colored(f"--- {summary.packages_skipped} package{plural} skipped ---", "yellow"))
    if summary.files_checked:
        plural = "" if summary.files_checked == 1 else "s"
        print(colored(f"--- success, {summary.files_checked} file{plural} checked ---", "green"))
    else:
        print_error("--- nothing to do; exit 1 ---")
        sys.exit(1)


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print_error("\n\nTest aborted due to KeyboardInterrupt!")
        sys.exit(1)



from collections.abc import Generator, Iterable

from ..core import _D, OrthodoxCalendar

class Bulgaria(OrthodoxCalendar):
    def get_shifted_holidays(self, days: Iterable[tuple[_D, str]]) -> Generator[tuple[_D, str]]: ...



import collections
import datetime
import re
from _typeshed import Incomplete
from collections.abc import Callable, Generator
from io import StringIO
from typing import Any, Final, Literal, overload

from dateparser.conf import Settings

NSP_COMPATIBLE: Final[re.Pattern[str]]
MERIDIAN: Final[re.Pattern[str]]
MICROSECOND: Final[re.Pattern[str]]
EIGHT_DIGIT: Final[re.Pattern[str]]
HOUR_MINUTE_REGEX: Final[re.Pattern[str]]

def no_space_parser_eligibile(datestring: str) -> bool: ...
def get_unresolved_attrs(
    parser_object: object,
) -> tuple[list[Literal["year", "month", "day"]], list[Literal["year", "month", "day"]]]: ...

date_order_chart: Final[dict[str, str]]

@overload
def resolve_date_order(order: str, lst: Literal[True]) -> list[str]: ...
@overload
def resolve_date_order(order: str, lst: Literal[False] | None = None) -> str: ...

class _time_parser:
    time_directives: list[str]
    def __call__(self, timestring: str) -> datetime.time: ...

time_parser: _time_parser

class _no_spaces_parser:
    period: dict[str, list[str]]
    date_formats: dict[str, list[str]]
    def __init__(self, *args, **kwargs): ...
    @classmethod
    def parse(cls, datestring: str, settings: Settings) -> tuple[datetime.datetime, str]: ...

class _parser:
    alpha_directives: collections.OrderedDict[str, list[str]]
    num_directives: dict[str, list[str]]
    settings: Settings
    tokens: list[tuple[Incomplete, Incomplete]]
    filtered_tokens: list[tuple[Incomplete, Incomplete, int]]
    unset_tokens: list[Incomplete]
    day: int | None
    month: int | None
    year: int | None
    time: Callable[[], datetime.time] | None
    auto_order: list[str]
    ordered_num_directives: collections.OrderedDict[str, list[str]]
    def __init__(self, tokens, settings: Settings): ...
    @classmethod
    def parse(cls, datestring: str, settings: Settings, tz: datetime.tzinfo | None = None) -> tuple[Incomplete, Incomplete]: ...

class tokenizer:
    digits: Literal["0123456789:"]
    letters: Literal["abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"]
    instream: StringIO
    def __init__(self, ds: str) -> None: ...
    def tokenize(self) -> Generator[tuple[str, Literal[0, 1, 2]], Any, None]: ...



from _typeshed import Incomplete
from collections.abc import Callable
from logging import Logger
from typing import Literal

from pika import amqp_object, frame

LOGGER: Logger

def name_or_value(value: amqp_object.AMQPObject | frame.Frame | int | str) -> str: ...
def sanitize_prefix(function): ...
def check_for_prefix_and_key(function): ...

class CallbackManager:
    CALLS: str
    ARGUMENTS: str
    DUPLICATE_WARNING: str
    CALLBACK: str
    ONE_SHOT: str
    ONLY_CALLER: str
    def __init__(self) -> None: ...
    def add(
        self,
        prefix: str | int,
        key: str | object,
        callback: Callable[[Incomplete], Incomplete],
        one_shot: bool = True,
        only_caller: object | None = None,
        arguments=None,
    ) -> tuple[str | int, str | object]: ...
    def clear(self) -> None: ...
    def cleanup(self, prefix: str | int) -> bool: ...
    def pending(self, prefix: str | int, key: str | object) -> int | None: ...
    def process(self, prefix: str | int, key: str | object, caller, *args, **keywords) -> bool: ...
    def remove(
        self,
        prefix: str | int,
        key: str | object,
        callback_value: Callable[[Incomplete], Incomplete] | None = None,
        arguments=None,
    ) -> Literal[True]: ...
    def remove_all(self, prefix: str | int, key: str | object) -> None: ...



from collections.abc import Mapping
from datetime import datetime
from typing import Any, ClassVar, Generic, TypeVar, overload
from typing_extensions import deprecated

from django.db.models import Model, QuerySet

def format_datetime(value: datetime, datetime_format: str) -> str: ...

class Widget:
    coerce_to_string: bool
    def __init__(self, coerce_to_string: bool = True) -> None: ...
    def clean(self, value: Any, row: Mapping[str, Any] | None = None, **kwargs: Any) -> Any: ...
    @overload
    @deprecated("The 'obj' parameter is deprecated and will be removed in a future release.")
    def render(self, value: Any, obj: Model, **kwargs: Any) -> Any: ...
    @overload
    def render(self, value: Any, obj: None = None, **kwargs: Any) -> Any: ...

class NumberWidget(Widget):
    def is_empty(self, value: Any) -> bool: ...

class FloatWidget(NumberWidget): ...
class IntegerWidget(NumberWidget): ...
class DecimalWidget(NumberWidget): ...

class CharWidget(Widget):
    allow_blank: bool
    def __init__(self, coerce_to_string: bool = True, allow_blank: bool = True) -> None: ...

class BooleanWidget(Widget):
    TRUE_VALUES: ClassVar[list[str | int | bool]]
    FALSE_VALUES: ClassVar[list[str | int | bool]]
    NULL_VALUES: ClassVar[list[str | None]]
    def __init__(self, coerce_to_string: bool = True) -> None: ...

class DateWidget(Widget):
    formats: tuple[str, ...]
    def __init__(self, format: str | None = None, coerce_to_string: bool = True) -> None: ...

class DateTimeWidget(Widget):
    formats: tuple[str, ...]
    def __init__(self, format: str | None = None, coerce_to_string: bool = True) -> None: ...

class TimeWidget(Widget):
    formats: tuple[str, ...]
    def __init__(self, format: str | None = None, coerce_to_string: bool = True) -> None: ...

class DurationWidget(Widget): ...

class SimpleArrayWidget(Widget):
    separator: str
    def __init__(self, separator: str | None = None, coerce_to_string: bool = True) -> None: ...

class JSONWidget(Widget): ...

_ModelT = TypeVar("_ModelT", bound=Model)

class ForeignKeyWidget(Widget, Generic[_ModelT]):
    model: _ModelT
    field: str
    key_is_id: bool
    use_natural_foreign_keys: bool
    def __init__(
        self, model: _ModelT, field: str = "pk", use_natural_foreign_keys: bool = False, key_is_id: bool = False, **kwargs: Any
    ) -> None: ...
    def get_queryset(self, value: Any, row: Mapping[str, Any], *args: Any, **kwargs: Any) -> QuerySet[_ModelT]: ...
    def get_lookup_kwargs(self, value: Any, row: Mapping[str, Any] | None = None, **kwargs: Any) -> dict[str, Any]: ...

class ManyToManyWidget(Widget, Generic[_ModelT]):
    model: _ModelT
    separator: str
    field: str
    def __init__(self, model: _ModelT, separator: str | None = None, field: str | None = None, **kwargs: Any) -> None: ...



import random
import timeit
from _typeshed import ReadableBuffer
from collections.abc import Iterable
from hmac import compare_digest
from typing import Final, Literal, SupportsBytes, SupportsIndex, overload

from passlib.utils.compat import JYTHON as JYTHON

__all__ = [
    "JYTHON",
    "sys_bits",
    "unix_crypt_schemes",
    "rounds_cost_values",
    "consteq",
    "saslprep",
    "xor_bytes",
    "render_bytes",
    "is_same_codec",
    "is_ascii_safe",
    "to_bytes",
    "to_unicode",
    "to_native_str",
    "has_crypt",
    "test_crypt",
    "safe_crypt",
    "tick",
    "rng",
    "getrandbytes",
    "getrandstr",
    "generate_password",
    "is_crypt_handler",
    "is_crypt_context",
    "has_rounds_info",
    "has_salt_info",
]

sys_bits: Final[int]
unix_crypt_schemes: list[str]
rounds_cost_values: Final[list[str]]

class SequenceMixin:
    def __getitem__(self, idx): ...
    def __iter__(self): ...
    def __len__(self) -> int: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...

consteq = compare_digest

def str_consteq(left: str | bytes, right: str | bytes) -> bool: ...
def splitcomma(source: str, sep: str = ",") -> list[str]: ...
def saslprep(source: str, param: str = "value") -> str: ...
def render_bytes(source: str | bytes, *args: str | bytes) -> bytes: ...
def bytes_to_int(value: Iterable[SupportsIndex] | SupportsBytes | ReadableBuffer) -> int: ...
def int_to_bytes(value: int, count: SupportsIndex) -> bytes: ...
def xor_bytes(
    left: Iterable[SupportsIndex] | SupportsBytes | ReadableBuffer,
    right: Iterable[SupportsIndex] | SupportsBytes | ReadableBuffer,
) -> bytes: ...
def repeat_string(source: str | bytes, size: int) -> str | bytes: ...
def is_ascii_codec(codec: str) -> bool: ...
def is_same_codec(left: str, right: str) -> bool: ...
def is_ascii_safe(source: str | bytes) -> bool: ...
def to_bytes(source: str | bytes, encoding: str = "utf-8", param: str = "value", source_encoding: str | None = None) -> bytes: ...
def to_unicode(source: str | bytes, encoding: str = "utf-8", param: str = "value") -> str: ...
def to_native_str(source: str | bytes, encoding: str = "utf-8", param: str = "value") -> str: ...

has_crypt: bool

def safe_crypt(secret: str | bytes, hash: str | bytes) -> str | None: ...
def test_crypt(secret: str | bytes, hash: str) -> bool: ...

timer = timeit.default_timer
tick = timer
rng: random.Random

@overload
def getrandbytes(rng: random.Random, count: None) -> Literal[b""]: ...
@overload
def getrandbytes(rng: random.Random, count) -> bytes: ...
def getrandstr(rng: random.Random, charset: str, count: int) -> str: ...
def generate_password(size: int = 10, charset: str = ...) -> str: ...
def is_crypt_handler(obj) -> bool: ...
def is_crypt_context(obj) -> bool: ...
def has_rounds_info(handler) -> bool: ...
def has_salt_info(handler) -> bool: ...



from __future__ import annotations

from typing import Any

from reportlab.lib import colors
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.units import inch
from reportlab.platypus.flowables import Image
from reportlab.platypus.paragraph import Paragraph
from reportlab.platypus.tables import Table, TableStyle

data: list[list[Any]]

# Verify all the examples from the docs work

#
# TableStyle User Methods
#
LIST_STYLE = TableStyle(
    [
        ("LINEABOVE", (0, 0), (-1, 0), 2, colors.green),
        ("LINEABOVE", (0, 1), (-1, -1), 0.25, colors.black),
        ("LINEBELOW", (0, -1), (-1, -1), 2, colors.green),
        ("ALIGN", (1, 1), (-1, -1), "RIGHT"),
    ]
)
LIST_STYLE.add("BACKGROUND", (0, 0), (-1, 0), colors.Color(0, 0.7, 0.7))

#
# TableStyle Cell Formatting Commands
#
data = [
    ["00", "01", "02", "03", "04"],
    ["10", "11", "12", "13", "14"],
    ["20", "21", "22", "23", "24"],
    ["30", "31", "32", "33", "34"],
]
t = Table(data)
t.setStyle(TableStyle([("BACKGROUND", (1, 1), (-2, -2), colors.green), ("TEXTCOLOR", (0, 0), (1, -1), colors.red)]))

data = [
    ["00", "01", "02", "03", "04"],
    ["10", "11", "12", "13", "14"],
    ["20", "21", "22", "23", "24"],
    ["30", "31", "32", "33", "34"],
]
t = Table(data, 5 * [0.4 * inch], 4 * [0.4 * inch])
# NOTE: I've modified this example to drop the optional TableStyle
#       wrapper, so we test both variants
t.setStyle(
    [
        ("ALIGN", (1, 1), (-2, -2), "RIGHT"),
        ("TEXTCOLOR", (1, 1), (-2, -2), colors.red),
        ("VALIGN", (0, 0), (0, -1), "TOP"),
        ("TEXTCOLOR", (0, 0), (0, -1), colors.blue),
        ("ALIGN", (0, -1), (-1, -1), "CENTER"),
        ("VALIGN", (0, -1), (-1, -1), "MIDDLE"),
        ("TEXTCOLOR", (0, -1), (-1, -1), colors.green),
        ("INNERGRID", (0, 0), (-1, -1), 0.25, colors.black),
        ("BOX", (0, 0), (-1, -1), 0.25, colors.black),
    ]
)

#
# Table Style Line Commands
#
data = [
    ["00", "01", "02", "03", "04"],
    ["10", "11", "12", "13", "14"],
    ["20", "21", "22", "23", "24"],
    ["30", "31", "32", "33", "34"],
]
Table(
    data,
    style=[
        ("GRID", (1, 1), (-2, -2), 1, colors.green),
        ("BOX", (0, 0), (1, -1), 2, colors.red),
        ("LINEABOVE", (1, 2), (-2, 2), 1, colors.blue),
        ("LINEBEFORE", (2, 1), (2, -2), 1, colors.pink),
    ],
)

data = [
    ["00", "01", "02", "03", "04"],
    ["10", "11", "12", "13", "14"],
    ["20", "21", "22", "23", "24"],
    ["30", "31", "32", "33", "34"],
]
Table(
    data,
    style=[
        ("GRID", (0, 0), (-1, -1), 0.5, colors.grey),
        ("GRID", (1, 1), (-2, -2), 1, colors.green),
        ("BOX", (0, 0), (1, -1), 2, colors.red),
        ("BOX", (0, 0), (-1, -1), 2, colors.black),
        ("LINEABOVE", (1, 2), (-2, 2), 1, colors.blue),
        ("LINEBEFORE", (2, 1), (2, -2), 1, colors.pink),
        ("BACKGROUND", (0, 0), (0, 1), colors.pink),
        ("BACKGROUND", (1, 1), (1, 2), colors.lavender),
        ("BACKGROUND", (2, 2), (2, 3), colors.orange),
    ],
)

#
# Complex Cell Values
#
styleSheet = getSampleStyleSheet()
I = Image("foo.jpg")
I.drawHeight = 1.25 * inch * I.drawHeight / I.drawWidth
I.drawWidth = 1.25 * inch
P0 = Paragraph(
    """<b>A pa<font color=red>r</font>a<i>graph</i></b>
    <super><font color=yellow>1</font></super>""",
    styleSheet["BodyText"],
)
P = Paragraph(
    """<para align=center spaceb=3>The <b>ReportLab Left
    <font color=red>Logo</font></b>
    Image</para>""",
    styleSheet["BodyText"],
)
data = [
    ["A", "B", "C", P0, "D"],
    ["00", "01", "02", [I, P], "04"],
    ["10", "11", "12", [P, I], "14"],
    ["20", "21", "22", "23", "24"],
    ["30", "31", "32", "33", "34"],
]
Table(
    data,
    style=[
        ("GRID", (1, 1), (-2, -2), 1, colors.green),
        ("BOX", (0, 0), (1, -1), 2, colors.red),
        ("LINEABOVE", (1, 2), (-2, 2), 1, colors.blue),
        ("LINEBEFORE", (2, 1), (2, -2), 1, colors.pink),
        ("BACKGROUND", (0, 0), (0, 1), colors.pink),
        ("BACKGROUND", (1, 1), (1, 2), colors.lavender),
        ("BACKGROUND", (2, 2), (2, 3), colors.orange),
        ("BOX", (0, 0), (-1, -1), 2, colors.black),
        ("GRID", (0, 0), (-1, -1), 0.5, colors.black),
        ("VALIGN", (3, 0), (3, 0), "BOTTOM"),
        ("BACKGROUND", (3, 0), (3, 0), colors.limegreen),
        ("BACKGROUND", (3, 1), (3, 1), colors.khaki),
        ("ALIGN", (3, 1), (3, 1), "CENTER"),
        ("BACKGROUND", (3, 2), (3, 2), colors.beige),
        ("ALIGN", (3, 2), (3, 2), "LEFT"),
    ],
)

#
# TableStyle Span Commands
#
data = [
    ["Top\\nLeft", "", "02", "03", "04"],
    ["", "", "12", "13", "14"],
    ["20", "21", "22", "Bottom\\nRight", ""],
    ["30", "31", "32", "", ""],
]
Table(
    data,
    style=[
        ("GRID", (0, 0), (-1, -1), 0.5, colors.grey),
        ("BACKGROUND", (0, 0), (1, 1), colors.palegreen),
        ("SPAN", (0, 0), (1, 1)),
        ("BACKGROUND", (-2, -2), (-1, -1), colors.pink),
        ("SPAN", (-2, -2), (-1, -1)),
    ],
)

#
# TableStyle Miscellaneous Commands
#
# NOTE: This one doesn't provide any actual examples, we just
#       make sure these pass into TableStyle/Table when mixed
#       with other commands
TableStyle([("NOSPLIT", (0, 0), (1, 1))])
LIST_STYLE.add("NOSPLIT", (0, 0), (1, 1))

TableStyle([("ROUNDEDCORNERS", [0, 0, 5, 5])])
TableStyle([("ROUNDEDCORNERS", (0, 0, 5, 5))])
LIST_STYLE.add("ROUNDEDCORNERS", [0, 0, 5, 5])
LIST_STYLE.add("ROUNDEDCORNERS", (0, 0, 5, 5))

Table(
    [["foo"]],
    style=[
        ("GRID", (0, 0), (-1, -1), 0.5, colors.grey),
        ("BACKGROUND", (0, 0), (0, 1), colors.pink),
        ("NOSPLIT", (0, 0), (1, 1)),
        ("ROUNDEDCORNERS", [0, 0, 5, 5]),
    ],
)


# Testing the various possible data layouts
Table([["foo"]])
Table([("foo",)])
Table((["foo"],))
Table((("foo",),))



from collections.abc import Iterable, Sequence

def bytes_to_bits() -> list[str]: ...

BYTES_TO_BITS: list[str]

def valid_words(words: Iterable[int], word_size: int, num_words: int) -> bool: ...
def int_to_words(int_val: int, word_size: int, num_words: int) -> tuple[int, ...]: ...
def words_to_int(words: Sequence[int], word_size: int, num_words: int) -> int: ...
def valid_bits(bits: str, width: int, word_sep: str = "") -> bool: ...
def bits_to_int(bits: str, width: int, word_sep: str = "") -> int: ...
def int_to_bits(int_val: int, word_size: int, num_words: int, word_sep: str = "") -> str: ...
def valid_bin(bin_val: str, width: int) -> bool: ...
def int_to_bin(int_val: int, width: int) -> str: ...
def bin_to_int(bin_val: str, width: int) -> int: ...



from collections.abc import Callable, Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["bfs_beam_edges"]

@_dispatchable
def bfs_beam_edges(
    G: Graph[_Node], source: _Node, value: Callable[[_Node], float], width: int | None = None
) -> Generator[tuple[_Node, _Node]]: ...



from typing import ClassVar
from typing_extensions import Self

import passlib.utils.handlers as uh
from passlib.handlers.misc import plaintext
from passlib.utils.handlers import PrefixWrapper

__all__ = [
    "ldap_plaintext",
    "ldap_md5",
    "ldap_sha1",
    "ldap_salted_md5",
    "ldap_salted_sha1",
    "ldap_salted_sha256",
    "ldap_salted_sha512",
    "ldap_des_crypt",
    "ldap_bsdi_crypt",
    "ldap_md5_crypt",
    "ldap_sha1_crypt",
    "ldap_bcrypt",
    "ldap_sha256_crypt",
    "ldap_sha512_crypt",
]

class _Base64DigestHelper(uh.StaticHandler):
    ident: ClassVar[str | None]
    checksum_chars: ClassVar[str]

class _SaltedBase64DigestHelper(uh.HasRawSalt, uh.HasRawChecksum, uh.GenericHandler):
    checksum_chars: ClassVar[str]
    ident: ClassVar[str | None]
    min_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    default_salt_size: ClassVar[int]
    @classmethod
    def from_string(cls, hash) -> Self: ...  # type: ignore[override]

class ldap_md5(_Base64DigestHelper):
    name: ClassVar[str]
    ident: ClassVar[str]

class ldap_sha1(_Base64DigestHelper):
    name: ClassVar[str]
    ident: ClassVar[str]

class ldap_salted_md5(_SaltedBase64DigestHelper):
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]

class ldap_salted_sha1(_SaltedBase64DigestHelper):
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]

class ldap_salted_sha256(_SaltedBase64DigestHelper):
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]
    default_salt_size: ClassVar[int]

class ldap_salted_sha512(_SaltedBase64DigestHelper):
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]
    default_salt_size: ClassVar[int]

class ldap_plaintext(plaintext):
    name: ClassVar[str]
    @classmethod
    def genconfig(cls): ...  # type: ignore[override]
    @classmethod
    def identify(cls, hash): ...

# Dynamically created
ldap_sha512_crypt: PrefixWrapper
ldap_sha256_crypt: PrefixWrapper
ldap_sha1_crypt: PrefixWrapper
ldap_bcrypt: PrefixWrapper
ldap_md5_crypt: PrefixWrapper
ldap_bsdi_crypt: PrefixWrapper
ldap_des_crypt: PrefixWrapper



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import typing

import google.protobuf.descriptor
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ReaderBaseState(google.protobuf.message.Message):
    """For serializing and restoring the state of ReaderBase, see
    reader_base.h for details.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WORK_STARTED_FIELD_NUMBER: builtins.int
    WORK_FINISHED_FIELD_NUMBER: builtins.int
    NUM_RECORDS_PRODUCED_FIELD_NUMBER: builtins.int
    CURRENT_WORK_FIELD_NUMBER: builtins.int
    work_started: builtins.int
    work_finished: builtins.int
    num_records_produced: builtins.int
    current_work: builtins.bytes
    def __init__(
        self,
        *,
        work_started: builtins.int | None = ...,
        work_finished: builtins.int | None = ...,
        num_records_produced: builtins.int | None = ...,
        current_work: builtins.bytes | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "current_work",
            b"current_work",
            "num_records_produced",
            b"num_records_produced",
            "work_finished",
            b"work_finished",
            "work_started",
            b"work_started",
        ],
    ) -> None: ...

global___ReaderBaseState = ReaderBaseState



import sys
from typing import Any, overload

_defaultaction: str
_onceregistry: dict[Any, Any]
filters: list[tuple[str, str | None, type[Warning], str | None, int]]

if sys.version_info >= (3, 12):
    @overload
    def warn(
        message: str,
        category: type[Warning] | None = None,
        stacklevel: int = 1,
        source: Any | None = None,
        *,
        skip_file_prefixes: tuple[str, ...] = (),
    ) -> None: ...
    @overload
    def warn(
        message: Warning,
        category: Any = None,
        stacklevel: int = 1,
        source: Any | None = None,
        *,
        skip_file_prefixes: tuple[str, ...] = (),
    ) -> None: ...

else:
    @overload
    def warn(message: str, category: type[Warning] | None = None, stacklevel: int = 1, source: Any | None = None) -> None: ...
    @overload
    def warn(message: Warning, category: Any = None, stacklevel: int = 1, source: Any | None = None) -> None: ...

@overload
def warn_explicit(
    message: str,
    category: type[Warning],
    filename: str,
    lineno: int,
    module: str | None = ...,
    registry: dict[str | tuple[str, type[Warning], int], int] | None = None,
    module_globals: dict[str, Any] | None = None,
    source: Any | None = None,
) -> None: ...
@overload
def warn_explicit(
    message: Warning,
    category: Any,
    filename: str,
    lineno: int,
    module: str | None = None,
    registry: dict[str | tuple[str, type[Warning], int], int] | None = None,
    module_globals: dict[str, Any] | None = None,
    source: Any | None = None,
) -> None: ...



from .errors import JoseError as JoseError
from .rfc7515 import (
    JsonWebSignature as JsonWebSignature,
    JWSAlgorithm as JWSAlgorithm,
    JWSHeader as JWSHeader,
    JWSObject as JWSObject,
)
from .rfc7516 import (
    JsonWebEncryption as JsonWebEncryption,
    JWEAlgorithm as JWEAlgorithm,
    JWEEncAlgorithm as JWEEncAlgorithm,
    JWEZipAlgorithm as JWEZipAlgorithm,
)
from .rfc7517 import JsonWebKey as JsonWebKey, Key as Key, KeySet as KeySet
from .rfc7518 import ECKey as ECKey, OctKey as OctKey, RSAKey as RSAKey
from .rfc7519 import BaseClaims as BaseClaims, JsonWebToken as JsonWebToken, JWTClaims as JWTClaims
from .rfc8037 import OKPKey as OKPKey

jwt: JsonWebToken

__all__ = [
    "JoseError",
    "JsonWebSignature",
    "JWSAlgorithm",
    "JWSHeader",
    "JWSObject",
    "JsonWebEncryption",
    "JWEAlgorithm",
    "JWEEncAlgorithm",
    "JWEZipAlgorithm",
    "JsonWebKey",
    "Key",
    "KeySet",
    "OctKey",
    "RSAKey",
    "ECKey",
    "OKPKey",
    "JsonWebToken",
    "BaseClaims",
    "JWTClaims",
    "jwt",
]



import xml.etree.ElementTree as ET
from _typeshed import Unused
from collections.abc import Callable
from typing import Any, Literal, overload

def current_global_nsmap() -> dict[str, str]: ...

class IncrementalTree(ET.ElementTree):
    def write(  # type: ignore[override]
        self,
        file_or_filename: ET._FileWrite,
        encoding: str | None = None,
        xml_declaration: bool | None = None,
        default_namespace: str | None = None,
        method: Literal["xml", "html", "text"] | None = None,  # does not accept 'c14n', unlike parent method
        *,
        short_empty_elements: bool = True,
        nsmap: dict[str, str] | None = None,
        root_ns_only: bool = False,
        minimal_ns_only: bool = False,
    ) -> None: ...

def process_attribs(
    elem: ET.Element[Any],
    is_nsmap_scope_changed: bool | None,
    default_ns_attr_prefix: str | None,
    nsmap_scope: dict[str, str],
    global_nsmap: dict[str, str],
    new_namespace_prefixes: set[str],
    uri_to_prefix: dict[str, str],
) -> tuple[list[tuple[str, str]], str | None, dict[str, str]]: ...
def write_elem_start(
    write: Callable[..., None],
    elem: ET.Element[Any],
    nsmap_scope: dict[str, str],
    global_nsmap: dict[str, str],
    short_empty_elements: bool | None,
    is_html: bool | None,
    is_root: bool = False,
    uri_to_prefix: dict[str, str] | None = None,
    default_ns_attr_prefix: str | None = None,
    new_nsmap: dict[str, str] | None = None,
    **kwargs: Unused,
) -> tuple[str | None, dict[str, str], str | None, dict[str, str] | None, bool]: ...
@overload
def tostring(
    element: ET.Element[Any],
    encoding: None = None,
    method: Literal["xml", "html", "text"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
    nsmap: dict[str, str] | None = None,
    root_ns_only: bool = False,
    minimal_ns_only: bool = False,
    tree_cls: type[ET.ElementTree] = ...,
) -> bytes: ...
@overload
def tostring(
    element: ET.Element[Any],
    encoding: Literal["unicode"],
    method: Literal["xml", "html", "text"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
    nsmap: dict[str, str] | None = None,
    root_ns_only: bool = False,
    minimal_ns_only: bool = False,
    tree_cls: type[ET.ElementTree] = ...,
) -> str: ...
@overload
def tostring(
    element: ET.Element[Any],
    encoding: str,
    method: Literal["xml", "html", "text"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
    nsmap: dict[str, str] | None = None,
    root_ns_only: bool = False,
    minimal_ns_only: bool = False,
    tree_cls: type[ET.ElementTree] = ...,
) -> Any: ...
@overload
def tostringlist(
    element: ET.Element[Any],
    encoding: None = None,
    method: Literal["xml", "html", "text"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
    nsmap: dict[str, str] | None = None,
    root_ns_only: bool = False,
    minimal_ns_only: bool = False,
    tree_cls: type[ET.ElementTree] = ...,
) -> list[bytes]: ...
@overload
def tostringlist(
    element: ET.Element[Any],
    encoding: Literal["unicode"],
    method: Literal["xml", "html", "text"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
    nsmap: dict[str, str] | None = None,
    root_ns_only: bool = False,
    minimal_ns_only: bool = False,
    tree_cls: type[ET.ElementTree] = ...,
) -> list[str]: ...
@overload
def tostringlist(
    element: ET.Element[Any],
    encoding: str,
    method: Literal["xml", "html", "text"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
    nsmap: dict[str, str] | None = None,
    root_ns_only: bool = False,
    minimal_ns_only: bool = False,
    tree_cls: type[ET.ElementTree] = ...,
) -> list[Any]: ...
@overload
def compat_tostring(
    element: ET.Element[Any],
    encoding: None = None,
    method: Literal["xml", "html", "text"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
    nsmap: dict[str, str] | None = None,
    root_ns_only: bool = True,
    minimal_ns_only: bool = False,
    tree_cls: type[ET.ElementTree] = ...,
) -> bytes: ...
@overload
def compat_tostring(
    element: ET.Element[Any],
    encoding: Literal["unicode"],
    method: Literal["xml", "html", "text"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
    nsmap: dict[str, str] | None = None,
    root_ns_only: bool = True,
    minimal_ns_only: bool = False,
    tree_cls: type[ET.ElementTree] = ...,
) -> str: ...
@overload
def compat_tostring(
    element: ET.Element[Any],
    encoding: str,
    method: Literal["xml", "html", "text"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
    nsmap: dict[str, str] | None = None,
    root_ns_only: bool = True,
    minimal_ns_only: bool = False,
    tree_cls: type[ET.ElementTree] = ...,
) -> Any: ...



import sys

if sys.platform == "darwin":
    from _typeshed import StrOrBytesPath
    from collections.abc import Sequence
    from socket import AddressFamily, SocketKind
    from typing import Final, TypeVar

    _T = TypeVar("_T")

    version: Final[int]
    SIDL: Final = 1
    SRUN: Final = 2
    SSLEEP: Final = 3
    SSTOP: Final = 4
    SZOMB: Final = 5
    TCPS_CLOSED: Final = 0
    TCPS_CLOSING: Final = 7
    TCPS_CLOSE_WAIT: Final = 5
    TCPS_LISTEN: Final = 1
    TCPS_ESTABLISHED: Final = 4
    TCPS_SYN_SENT: Final = 2
    TCPS_SYN_RECEIVED: Final = 3
    TCPS_FIN_WAIT_1: Final = 6
    TCPS_FIN_WAIT_2: Final = 9
    TCPS_LAST_ACK: Final = 8
    TCPS_TIME_WAIT: Final = 10
    PSUTIL_CONN_NONE: Final = 128

    def proc_cmdline(pid: int, /) -> list[str]: ...
    def proc_cwd(pid: int, /) -> str: ...
    def proc_environ(pid: int, /) -> str: ...
    def proc_exe(pid: int, /) -> str: ...
    def proc_kinfo_oneshot(pid: int, /) -> tuple[int, int, int, int, int, int, int, float, int, str]: ...
    def proc_memory_uss(pid: int, /) -> int: ...
    def proc_name(pid: int, /) -> str: ...
    def proc_net_connections(
        pid: int, af_filter: Sequence[AddressFamily | int | None], type_filter: Sequence[SocketKind | int | None], /
    ) -> list[
        tuple[int, int, int, tuple[str | None, int], tuple[str | None, int] | tuple[()], int]
        | tuple[int, int, int, str, str, int]
    ]: ...
    def proc_num_fds(pid: int, /) -> int: ...
    def proc_open_files(pid: int, /) -> list[tuple[str, int]]: ...
    def proc_pidtaskinfo_oneshot(pid: int, /) -> tuple[float, float, int, int, int, int, int, int]: ...
    def proc_threads(pid: int, /) -> list[tuple[int, float, float]]: ...
    def boot_time() -> float: ...
    def cpu_count_cores() -> int | None: ...
    def cpu_count_logical() -> int | None: ...
    def cpu_freq() -> tuple[int, int, int]: ...
    def cpu_stats() -> tuple[int, int, int, int, int]: ...
    def cpu_times() -> tuple[float, float, float, float]: ...
    def disk_io_counters() -> dict[str, tuple[int, int, int, int, int, int]]: ...
    def disk_partitions() -> list[tuple[str, str, str, str]]: ...
    def disk_usage_used(mount_point: StrOrBytesPath, default: _T, /) -> int | _T: ...
    def net_io_counters() -> dict[str, tuple[int, int, int, int, int, int, int, int]]: ...
    def per_cpu_times() -> list[tuple[float, float, float, float]]: ...
    def pids() -> list[int]: ...
    def sensors_battery() -> tuple[int, int, int]: ...
    def swap_mem() -> tuple[int, int, int, int, int]: ...
    def users() -> list[tuple[str, str, str, float, int]]: ...
    def virtual_mem() -> tuple[int, int, int, int, int, int]: ...
    def check_pid_range(pid: int, /) -> None: ...
    def set_debug(value: bool, /) -> None: ...



from _typeshed import Incomplete

from win32comext.axdebug import gateways

class DebugCodeContext(gateways.DebugCodeContext, gateways.DebugDocumentContext):
    debugSite: Incomplete
    offset: Incomplete
    length: Incomplete
    breakPointState: int
    lineno: Incomplete
    codeContainer: Incomplete
    def __init__(self, lineNo, charPos, len, codeContainer, debugSite) -> None: ...
    def GetDocumentContext(self): ...
    def SetBreakPoint(self, bps) -> None: ...
    def GetDocument(self): ...
    def EnumCodeContexts(self): ...

class EnumDebugCodeContexts(gateways.EnumDebugCodeContexts): ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class HistogramProto(google.protobuf.message.Message):
    """Serialization format for histogram module in
    tsl/lib/histogram/histogram.h
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MIN_FIELD_NUMBER: builtins.int
    MAX_FIELD_NUMBER: builtins.int
    NUM_FIELD_NUMBER: builtins.int
    SUM_FIELD_NUMBER: builtins.int
    SUM_SQUARES_FIELD_NUMBER: builtins.int
    BUCKET_LIMIT_FIELD_NUMBER: builtins.int
    BUCKET_FIELD_NUMBER: builtins.int
    min: builtins.float
    max: builtins.float
    num: builtins.float
    sum: builtins.float
    sum_squares: builtins.float
    @property
    def bucket_limit(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]:
        """Parallel arrays encoding the bucket boundaries and the bucket values.
        bucket(i) is the count for the bucket i.  The range for
        a bucket is:
          i == 0:  -DBL_MAX .. bucket_limit(0)
          i != 0:  bucket_limit(i-1) .. bucket_limit(i)
        """

    @property
    def bucket(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        min: builtins.float | None = ...,
        max: builtins.float | None = ...,
        num: builtins.float | None = ...,
        sum: builtins.float | None = ...,
        sum_squares: builtins.float | None = ...,
        bucket_limit: collections.abc.Iterable[builtins.float] | None = ...,
        bucket: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "bucket",
            b"bucket",
            "bucket_limit",
            b"bucket_limit",
            "max",
            b"max",
            "min",
            b"min",
            "num",
            b"num",
            "sum",
            b"sum",
            "sum_squares",
            b"sum_squares",
        ],
    ) -> None: ...

global___HistogramProto = HistogramProto



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2020 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class TaskDeviceFilters(google.protobuf.message.Message):
    """This file contains protos to be used when defining a TensorFlow
    cluster.

    Configure device filters for remote tasks in the cluster. When associated
    with a ClusterDef in setting up the cluster, a remote task will ignore all
    devices which do not match any of its filters. Device filters must be
    configured at the cluster startup, and cannot be updated once the cluster is
    up and running.

    EXAMPLES
    --------

    A two-job cluster with the following ClusterDef:

     Cluster:
       job { name: 'worker' tasks { key: 0 value: 'worker1:2222' }
                            tasks { key: 1 value: 'worker2:2222' } }
       job { name: 'ps'     tasks { key: 0 value: 'ps0:2222' }
                            tasks { key: 1 value: 'ps1:2222' } }

    Set device filters to isolate worker tasks:

     ClusterDeviceFilters:
       job { name: 'worker' tasks { key: 0
                                    value: device_filter '/job:ps'
                                           device_filter '/job:worker/task:0' }
                            tasks { key: 1
                                    value: device_filter '/job:ps'
                                           device_filter '/job:worker/task:1' } }

    Defines the device filters for a remote task.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEVICE_FILTERS_FIELD_NUMBER: builtins.int
    @property
    def device_filters(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(self, *, device_filters: collections.abc.Iterable[builtins.str] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["device_filters", b"device_filters"]) -> None: ...

global___TaskDeviceFilters = TaskDeviceFilters

@typing.final
class JobDeviceFilters(google.protobuf.message.Message):
    """Defines the device filters for tasks in a job."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class TasksEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        @property
        def value(self) -> global___TaskDeviceFilters: ...
        def __init__(self, *, key: builtins.int | None = ..., value: global___TaskDeviceFilters | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    TASKS_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The name of this job."""
    @property
    def tasks(self) -> google.protobuf.internal.containers.MessageMap[builtins.int, global___TaskDeviceFilters]:
        """Mapping from task ID to task device filters."""

    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        tasks: collections.abc.Mapping[builtins.int, global___TaskDeviceFilters] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["name", b"name", "tasks", b"tasks"]) -> None: ...

global___JobDeviceFilters = JobDeviceFilters

@typing.final
class ClusterDeviceFilters(google.protobuf.message.Message):
    """Defines the device filters for jobs in a cluster."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOBS_FIELD_NUMBER: builtins.int
    @property
    def jobs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___JobDeviceFilters]: ...
    def __init__(self, *, jobs: collections.abc.Iterable[global___JobDeviceFilters] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["jobs", b"jobs"]) -> None: ...

global___ClusterDeviceFilters = ClusterDeviceFilters



import re
from typing import Final

__version__: Final[str]
__author__: Final[str]
__email__: Final[str]
RFC3339_REGEX_FLAGS: Final[int]
RFC3339_REGEX: Final[re.Pattern[str]]

def validate_rfc3339(date_string: str) -> bool: ...



from _typeshed import Incomplete

from braintree.customer import Customer
from braintree.error_result import ErrorResult
from braintree.resource_collection import ResourceCollection
from braintree.successful_result import SuccessfulResult

class CustomerGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def all(self) -> ResourceCollection: ...
    def create(self, params: dict[str, Incomplete] | None = None) -> SuccessfulResult | ErrorResult | None: ...
    def delete(self, customer_id: str) -> SuccessfulResult: ...
    def find(self, customer_id: str, association_filter_id: str | None = None) -> Customer: ...
    def search(self, *query) -> ResourceCollection: ...
    def update(self, customer_id: str, params: dict[str, Incomplete] | None = None) -> SuccessfulResult | ErrorResult | None: ...



from _typeshed import Incomplete, SupportsKeysAndGetItem, SupportsNext, SupportsRichComparison
from collections.abc import Generator, Iterable, Iterator, Mapping, MutableMapping
from typing import Literal, TypeVar, overload

_T = TypeVar("_T")

class URIDict(MutableMapping[str, str]):
    def normalize(self, uri: str) -> str: ...
    store: dict[str, str]
    def __init__(self, m: SupportsKeysAndGetItem[str, str] | Iterable[tuple[str, str]], /, **kwargs: str) -> None: ...
    def __getitem__(self, uri: str) -> str: ...
    def __setitem__(self, uri: str, value: str) -> None: ...
    def __delitem__(self, uri: str) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...

class Unset: ...

def format_as_index(container: str, indices: Iterable[Incomplete] | None) -> str: ...
def find_additional_properties(instance: Iterable[str], schema: Mapping[str, Iterable[str]]) -> Generator[str]: ...
def extras_msg(extras: Iterable[object]) -> tuple[str, Literal["was", "were"]]: ...  # elements are passed to the repr() function
@overload
def ensure_list(thing: str) -> list[str]: ...
@overload
def ensure_list(thing: _T) -> _T: ...
def equal(one, two) -> bool: ...
def unbool(element, true=..., false=...): ...
def uniq(container: Iterable[SupportsRichComparison]) -> bool: ...
def find_evaluated_item_indexes_by_schema(validator, instance, schema) -> list[Incomplete]: ...
def find_evaluated_property_keys_by_schema(validator, instance, schema) -> list[Incomplete]: ...
def is_valid(errs_it: SupportsNext[object]) -> bool: ...



import datetime

class DayCount:
    epoch: float
    def __init__(self, epoch: float) -> None: ...
    def to_gregorian(self, dc: int) -> tuple[int, int, int]: ...
    def from_gregorian(self, year: int, month: int, day: int) -> float: ...
    def to_jd(self, dc: int) -> float: ...
    def from_jd(self, jdc: float) -> float: ...
    def from_julian(self, year: int, month: int, day: int) -> float: ...
    def to_julian(self, dc: int) -> tuple[int, int, int]: ...
    def to_datetime(self, dc: int) -> datetime.datetime: ...
    def from_datetime(self, date: datetime.datetime) -> float: ...



from collections.abc import Callable, Iterator, MutableMapping
from typing import Any, NamedTuple

from jwcrypto.jwe import JWE
from jwcrypto.jws import JWS

def base64url_encode(payload: str | bytes) -> str: ...
def base64url_decode(payload: str) -> bytes: ...
def json_encode(string: str | bytes) -> str: ...

# The function returns json.loads which returns Any
def json_decode(string: str | bytes) -> Any: ...

class JWException(Exception): ...

class InvalidJWAAlgorithm(JWException):
    def __init__(self, message: str | None = None) -> None: ...

class InvalidCEKeyLength(JWException):
    def __init__(self, expected: int, obtained: int) -> None: ...

class InvalidJWEOperation(JWException):
    def __init__(self, message: str | None = None, exception: BaseException | None = None) -> None: ...

class InvalidJWEKeyType(JWException):
    def __init__(self, expected: int, obtained: int) -> None: ...

class InvalidJWEKeyLength(JWException):
    def __init__(self, expected: int, obtained: int) -> None: ...

class InvalidJWSERegOperation(JWException):
    def __init__(self, message: str | None = None, exception: BaseException | None = None) -> None: ...

class JWKeyNotFound(JWException):
    def __init__(self, message: str | None = None) -> None: ...

class JWSEHeaderParameter(NamedTuple):
    description: str
    mustprotect: bool
    supported: bool
    check_fn: Callable[[JWS | JWE], bool] | None

class JWSEHeaderRegistry(MutableMapping[str, JWSEHeaderParameter]):
    def __init__(self, init_registry: dict[str, JWSEHeaderParameter] | None = None) -> None: ...
    def check_header(self, h: str, value: JWS | JWE) -> bool: ...
    def __getitem__(self, key: str) -> JWSEHeaderParameter: ...
    def __iter__(self) -> Iterator[str]: ...
    def __delitem__(self, key: str) -> None: ...
    def __setitem__(self, h: str, jwse_header_param: JWSEHeaderParameter) -> None: ...
    def __len__(self) -> int: ...
